@@ -26,6 +26,7 @@
 
 #include "address-book.h"
 #include "information.h"
+#include "misc.h"
 
 static GtkObjectClass *parent_class = NULL;
 
@@ -46,6 +47,7 @@ enum {
     STORE_ADDRESS,
     SAVE_CONFIG,
     LOAD_CONFIG,
+    ALIAS_COMPLETE,
     LAST_SIGNAL
 };
 
@@ -88,7 +90,8 @@ libbalsa_address_book_class_init(LibBalsaAddressBookClass * klass)
 		       GTK_RUN_FIRST,
 		       object_class->type,
 		       GTK_SIGNAL_OFFSET(LibBalsaAddressBookClass, load),
-		       gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
+		       gtk_marshal_NONE__POINTER_POINTER, GTK_TYPE_NONE, 2,
+		       GTK_TYPE_POINTER, GTK_TYPE_POINTER);
     libbalsa_address_book_signals[STORE_ADDRESS] =
 	gtk_signal_new("store-address",
 		       GTK_RUN_FIRST,
@@ -109,6 +112,12 @@ libbalsa_address_book_class_init(LibBalsaAddressBookClass * klass)
 					 load_config),
 		       gtk_marshal_NONE__POINTER, GTK_TYPE_NONE, 1,
 		       GTK_TYPE_POINTER);
+    libbalsa_address_book_signals[ALIAS_COMPLETE] =
+	gtk_signal_new("alias-complete", GTK_RUN_LAST, object_class->type,
+		       GTK_SIGNAL_OFFSET(LibBalsaAddressBookClass,
+					 alias_complete),
+		       libbalsa_marshall_POINTER__POINTER_POINTER, GTK_TYPE_POINTER, 2,
+		       GTK_TYPE_POINTER, GTK_TYPE_POINTER);
 
     gtk_object_class_add_signals(object_class,
 				 libbalsa_address_book_signals,
@@ -118,6 +127,7 @@ libbalsa_address_book_class_init(LibBalsaAddressBookClass * klass)
     klass->store_address = NULL;
     klass->save_config = libbalsa_address_book_real_save_config;
     klass->load_config = libbalsa_address_book_real_load_config;
+    klass->alias_complete = NULL;
 
     object_class->destroy = libbalsa_address_book_destroy;
 }
@@ -127,9 +137,10 @@ libbalsa_address_book_init(LibBalsaAddressBook * ab)
 {
     ab->config_prefix = NULL;
 
-    ab->address_list = NULL;
     ab->name = NULL;
     ab->expand_aliases = FALSE;
+    ab->dist_list_mode = FALSE;
+    ab->is_expensive   = FALSE;
 }
 
 static void
@@ -139,10 +150,6 @@ libbalsa_address_book_destroy(GtkObject * object)
 
     ab = LIBBALSA_ADDRESS_BOOK(object);
 
-    g_list_foreach(ab->address_list, (GFunc) gtk_object_unref, NULL);
-    g_list_free(ab->address_list);
-    ab->address_list = NULL;
-
     g_free(ab->config_prefix);
     ab->config_prefix = NULL;
 
@@ -160,7 +167,7 @@ libbalsa_address_book_new_from_config(const gchar * prefix)
     gchar *type_str;
     GtkType type;
     gboolean got_default;
-    LibBalsaAddressBook *address_book;
+    LibBalsaAddressBook *address_book = NULL;
 
     gnome_config_push_prefix(prefix);
     type_str = gnome_config_get_string_with_default("Type", &got_default);
@@ -168,6 +175,7 @@ libbalsa_address_book_new_from_config(const gchar * prefix)
     if (got_default == TRUE) {
 	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
 			     _("Cannot load address book %s"), prefix);
+	gnome_config_pop_prefix();
 	return NULL;
     }
 
@@ -176,20 +184,18 @@ libbalsa_address_book_new_from_config(const gchar * prefix)
 	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
 			     _("No such address book type: %s"), type_str);
 	g_free(type_str);
+	gnome_config_pop_prefix();
 	return NULL;
     }
 
     address_book = gtk_type_new(type);
     if (address_book == NULL) {
-	libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-			     _
-			     ("Could not create a address book of type %s"),
-			     type_str);
-	g_free(type_str);
-	return NULL;
-    }
-
-    libbalsa_address_book_load_config(address_book, prefix);
+	libbalsa_information
+	    (LIBBALSA_INFORMATION_WARNING,
+	     _("Could not create a address book of type %s"),
+	     type_str);
+    } else
+	libbalsa_address_book_load_config(address_book, prefix);
 
     gnome_config_pop_prefix();
     g_free(type_str);
@@ -199,11 +205,11 @@ libbalsa_address_book_new_from_config(const gchar * prefix)
 }
 
 void
-libbalsa_address_book_load(LibBalsaAddressBook * ab)
+libbalsa_address_book_load(LibBalsaAddressBook * ab, LibBalsaAddressBookLoadFunc callback, gpointer closure)
 {
     g_return_if_fail(LIBBALSA_IS_ADDRESS_BOOK(ab));
 
-    gtk_signal_emit(GTK_OBJECT(ab), libbalsa_address_book_signals[LOAD]);
+    gtk_signal_emit(GTK_OBJECT(ab), libbalsa_address_book_signals[LOAD], callback, closure);
 }
 
 void
@@ -242,6 +248,30 @@ libbalsa_address_book_load_config(LibBalsaAddressBook * ab,
     gnome_config_pop_prefix();
 }
 
+GList *
+libbalsa_address_book_alias_complete(LibBalsaAddressBook *ab,
+				     const gchar * prefix,
+				     gchar ** new_prefix)
+{
+    GList *res = NULL;
+
+    g_return_val_if_fail(LIBBALSA_IS_ADDRESS_BOOK(ab), NULL);
+
+    gtk_signal_emit(GTK_OBJECT(ab), libbalsa_address_book_signals[ALIAS_COMPLETE], prefix, new_prefix, &res);
+    return res;
+
+}
+
+
+gboolean libbalsa_address_is_dist_list(const LibBalsaAddressBook *ab,
+				       const LibBalsaAddress *address)
+{
+    return (address->member_list!=NULL || 
+	    ab->dist_list_mode && g_list_length(address->address_list)>1);
+}
+
+
+
 static void
 libbalsa_address_book_real_save_config(LibBalsaAddressBook * ab,
 				       const gchar * prefix)
@@ -251,6 +281,7 @@ libbalsa_address_book_real_save_config(LibBalsaAddressBook * ab,
     gnome_config_set_string("Type", gtk_type_name(GTK_OBJECT_TYPE(ab)));
     gnome_config_set_string("Name", ab->name);
     gnome_config_set_bool("ExpandAliases", ab->expand_aliases);
+    gnome_config_set_bool("DistListMode", ab->dist_list_mode);
 
     g_free(ab->config_prefix);
     ab->config_prefix = g_strdup(prefix);
@@ -266,7 +297,9 @@ libbalsa_address_book_real_load_config(LibBalsaAddressBook * ab,
     ab->config_prefix = g_strdup(prefix);
 
     ab->expand_aliases = gnome_config_get_bool("ExpandAliases=false");
+    ab->dist_list_mode = gnome_config_get_bool("DistListMode=false");
 
     g_free(ab->name);
     ab->name = gnome_config_get_string("Name=Address Book");
 }
+
