@@ -490,16 +490,12 @@ libbalsa_inputData_free(inputData * data)
  *     is a complete e-mail address.
  *************************************************************/
 static gboolean
-libbalsa_is_an_email(gchar *str) {
-    gboolean found;
-    unsigned i;
-    
-    found = FALSE;
-    for (i = 0; i < strlen(str); i++)
-	if ((str[i] == (gchar) '@') ||
-	    (str[i] == (gchar) '%') ||
-	    (str[i] == (gchar) '!')) found = TRUE;
-    return found;
+libbalsa_is_an_email(gchar * str)
+{
+    while (*str) 
+        if (*str == '@' || *str == '%' || *str++ == '!')
+            return TRUE;
+    return FALSE;
 }
 
 
@@ -638,14 +634,15 @@ libbalsa_move_forward_word(LibBalsaAddressEntry *address_entry)
 {
     inputData *input;
     emailData *addy;
+    size_t tmp;
 
     g_return_if_fail(address_entry != NULL);
     g_return_if_fail(LIBBALSA_IS_ADDRESS_ENTRY(address_entry));
 
     input = address_entry->input;
     addy = input->active->data;
-    if (addy->cursor < strlen(addy->user)) {
-	addy->cursor = strlen(addy->user);
+    if (addy->cursor < (tmp = strlen(addy->user))) {
+	addy->cursor = tmp;
     } else {
 	libbalsa_force_no_match(addy);
 	if (g_list_next(input->active)) {
@@ -715,6 +712,7 @@ libbalsa_fill_input(LibBalsaAddressEntry *address_entry)
     GList *list = NULL;
     emailData *addy;
     inputData *input;
+    size_t tmp;
 
     g_return_val_if_fail(address_entry != NULL, NULL);
     g_return_val_if_fail(LIBBALSA_IS_ADDRESS_ENTRY(address_entry), NULL);
@@ -778,7 +776,8 @@ libbalsa_fill_input(LibBalsaAddressEntry *address_entry)
     if (input->active != input->list)
 	addy->cursor = addy->cursor - 1; /* Compensate for the ',' */
     if (addy->cursor < 0) addy->cursor = 0;
-    if (addy->cursor > strlen(addy->user)) addy->cursor = strlen(addy->user);
+    if (addy->cursor > (tmp = strlen(addy->user)))
+        addy->cursor = tmp;
 
     return input;
 }
@@ -806,19 +805,16 @@ libbalsa_address_entry_set_text(LibBalsaAddressEntry *address,
     gint tmp_pos;
 
     GtkEditable *editable;
-    GtkEntry *entry;
 
     g_return_if_fail(address != NULL);
     g_return_if_fail(LIBBALSA_IS_ADDRESS_ENTRY(address));
     g_return_if_fail(text != NULL);
 
-    entry = GTK_ENTRY(address);
     editable = GTK_EDITABLE(address);
-    libbalsa_address_entry_delete_text(editable, 0, entry->text_length);
+    libbalsa_address_entry_delete_text(editable, 0, -1);
 
     tmp_pos = 0;
     gtk_editable_insert_text(editable, text, strlen(text), &tmp_pos);
-    editable->current_pos = tmp_pos;
 
     editable->selection_start_pos = 0;
     editable->selection_end_pos = 0;
@@ -877,7 +873,7 @@ libbalsa_delete_forward_word(LibBalsaAddressEntry *address_entry)
     /*
      * Lets see if the user is at the end of an e-mail entry.
      */
-    if (addy->cursor == strlen(addy->user)) {
+    if (addy->user[addy->cursor] == '\0') {
 	list = g_list_next(input->active);
 	if (list != NULL) {
 	    input->list = g_list_remove_link(input->list, list);
@@ -1222,7 +1218,7 @@ libbalsa_address_entry_delete_text(GtkEditable *editable,
 
     entry = GTK_ENTRY(editable);
 
-    if (end_pos < 0)
+    if (end_pos == (unsigned) -1)
 	end_pos = entry->text_length;
 
     if (editable->selection_start_pos > start_pos)
@@ -1613,7 +1609,7 @@ libbalsa_delete_backward_character(LibBalsaAddressEntry *address_entry)
        g_free(left);
        addy->user = str;
        addy->cursor--;
-       if (strlen(str) == 0)
+       if (*str == '\0')
 	   libbalsa_force_no_match(addy);
        else if (address_entry->find_match)
 	   (*address_entry->find_match) (addy, TRUE);
@@ -1769,15 +1765,15 @@ libbalsa_keystroke_enter(LibBalsaAddressEntry *address_entry)
      * Else no match was found.  Check if there was a default
      * domain to add to e-mail addresses.
      */
-    if (address_entry->domain == NULL) return;
-    if (strlen(address_entry->domain) == 0) return;
+    if (address_entry->domain == NULL || *address_entry->domain == '\0')
+	return;
 
     /*
      * There is a default domain to add.  Do we need to add it?
      */
     addy = address_entry->input->active->data;
-    if (libbalsa_is_an_email(addy->user)) return;
-    if (strlen(addy->user) == 0) return;
+    if (libbalsa_is_an_email(addy->user) || *addy->user == '\0')
+	return;
 
     /*
      * Okay, add it.
@@ -2012,9 +2008,10 @@ libbalsa_keystroke_comma(LibBalsaAddressEntry *address_entry)
     /*
      * And we add the default domain to the original entry.
      */
-    if (address_entry->domain == NULL) return;
-    if (strlen(address_entry->domain) == 0) return;
-    if (libbalsa_is_an_email(addy->user)) return;
+    if (address_entry->domain == NULL ||
+	*address_entry->domain == '\0' ||
+	libbalsa_is_an_email(addy->user)) return;
+
     str = g_strconcat(addy->user, "@", address_entry->domain, NULL);
     g_free(addy->user);
     addy->user = str;
@@ -2302,6 +2299,7 @@ libbalsa_cut_clipboard(LibBalsaAddressEntry *address_entry)
     gchar *str, *left, *right, *new;
     emailData *addy;
     gint i;
+    size_t tmp;
 
     g_return_if_fail(address_entry != NULL);
     g_return_if_fail(LIBBALSA_IS_ADDRESS_ENTRY(address_entry));
@@ -2339,8 +2337,8 @@ libbalsa_cut_clipboard(LibBalsaAddressEntry *address_entry)
 	g_free(addy->user);
 	addy->user = new;
 	addy->cursor = start_pos;
-	if (addy->cursor > strlen(addy->user))
-	    addy->cursor = strlen(addy->user);
+	if (addy->cursor > (tmp = strlen(addy->user)))
+	    addy->cursor = tmp;
 	if (addy->cursor < 0)
 	    addy->cursor = 0;
     } else {
@@ -2356,8 +2354,8 @@ libbalsa_cut_clipboard(LibBalsaAddressEntry *address_entry)
 	addy->user = left;
 	addy->cursor = start_pos;
 	address_entry->input->active = start;
-	if (addy->cursor > strlen(addy->user))
-	    addy->cursor = strlen(addy->user);
+	if (addy->cursor > (tmp = strlen(addy->user)))
+	    addy->cursor = tmp;
 	
 	/*
 	 * Set the end data.
@@ -2940,9 +2938,8 @@ libbalsa_address_entry_show(LibBalsaAddressEntry *address_entry)
     start = editable->selection_start_pos;
     end = editable->selection_end_pos;
     tmp_pos = 0;
-    libbalsa_address_entry_delete_text(editable, 0,
-	    GTK_ENTRY(address_entry)->text_length);
-    gtk_editable_insert_text(editable, show->str, strlen(show->str), &tmp_pos);
+    libbalsa_address_entry_delete_text(editable, 0, -1);
+    gtk_editable_insert_text(editable, show->str, show->len, &tmp_pos);
     gtk_editable_set_position(GTK_EDITABLE(address_entry), cursor);
     editable->selection_start_pos = start;
     editable->selection_end_pos = end;
@@ -2952,7 +2949,7 @@ libbalsa_address_entry_show(LibBalsaAddressEntry *address_entry)
 
 
 /*************************************************************
- * libbalsa_address_entry_show:
+ * libbalsa_address_clear_match:
  *     Clears the input cache of any data.
  *
  *   arguments:
