@@ -39,6 +39,7 @@ typedef enum _LibBalsaAddressField LibBalsaAddressField;
 enum _LibBalsaAddressField {
     FULL_NAME,
     FIRST_NAME,
+    MIDDLE_NAME,
     LAST_NAME,
     ORGANIZATION,
     EMAIL_ADDRESS,
@@ -61,6 +62,7 @@ struct _LibBalsaAddress {
      */
     gchar *full_name;
     gchar *first_name;
+    gchar *middle_name;
     gchar *last_name;
 
     /* Organisation
@@ -72,6 +74,7 @@ struct _LibBalsaAddress {
      * A list of user@domain.
      */
     GList *address_list;
+    GList *member_list;
 };
 
 struct _LibBalsaAddressClass {
@@ -79,14 +82,21 @@ struct _LibBalsaAddressClass {
 };
 
 GtkType libbalsa_address_get_type(void);
-
+ 
 LibBalsaAddress *libbalsa_address_new(void);
 LibBalsaAddress *libbalsa_address_new_from_string(gchar * address);
 GList *libbalsa_address_new_list_from_string(gchar * address);
 
-gchar *libbalsa_address_to_gchar(LibBalsaAddress * addr);
+gchar *libbalsa_address_to_gchar(LibBalsaAddress * addr, gint n);
 
 /* get pointer to descriptive name (full name if available, or e-mail) */
 const gchar *libbalsa_address_get_name(const LibBalsaAddress * addr);
 
+#if ENABLE_ESMTP
+/* XXX - added by Brian Stafford <brian@stafford.uklinux.net> */
+const gchar *libbalsa_address_get_phrase(LibBalsaAddress * address);
+/* XXX - added by Brian Stafford <brian@stafford.uklinux.net> */
+const gchar *libbalsa_address_get_mailbox(LibBalsaAddress * address, gint n);
+#endif
+
 #endif				/* __LIBBALSA_ADDRESS_H__ */
