@@ -23,6 +23,7 @@
 #include "config.h"
 #include <gnome.h>
 
+#include "misc.h"
 #include "files.h"
 
 static const gchar *permanent_prefixes[] = {
@@ -101,7 +102,9 @@ libbalsa_icon_finder(const char *mime_type, const char *filename)
     const char *content_type, *icon_file;
     gchar *icon = NULL;
     
-    content_type = (!mime_type) ? gnome_mime_type (filename) : mime_type;
+    content_type = (!mime_type) ? 
+	libbalsa_lookup_mime_type(filename) : 
+	mime_type;
 
     icon_file = gnome_mime_get_value (content_type, "icon-filename");
 
