@@ -27,15 +27,14 @@ typedef struct _LibBalsaMailbox LibBalsaMailbox;
 typedef struct _LibBalsaMailboxRemote LibBalsaMailboxRemote;
 typedef struct _LibBalsaMessage LibBalsaMessage;
 typedef struct _LibBalsaMessageBody LibBalsaMessageBody;
-typedef struct _ImapDir ImapDir;
 typedef struct _LibBalsaServer LibBalsaServer;
 
 /* Opaque structures */
 typedef struct body MuttBody;
+typedef struct header MuttHeader;
 
 #include "address.h"
 #include "message.h"
-#include "imapdir.h"
 #include "body.h"
 #include "files.h"
 #include "misc.h"
@@ -48,6 +47,7 @@ typedef struct body MuttBody;
 
 #include "address-book.h"
 #include "address-book-vcard.h"
+#include "address-book-ldif.h"
 #if ENABLE_LDAP
 #include "address-book-ldap.h"
 #endif
@@ -57,6 +57,9 @@ typedef struct body MuttBody;
 #include "mailbox_remote.h"
 #include "mailbox_pop3.h"
 #include "mailbox_imap.h"
+#include "mailbox_mbox.h"
+#include "mailbox_mh.h"
+#include "mailbox_maildir.h"
 
 #ifdef BALSA_SHOW_ALL
 #include "filter.h"
@@ -68,7 +71,11 @@ typedef struct body MuttBody;
 void libbalsa_init(LibBalsaInformationFunc information_callback);
 void libbalsa_set_spool(gchar * spool);
 
+void libbalsa_show_message_source(LibBalsaMessage* msg);
+
 gchar *libbalsa_guess_mail_spool(void);
+gboolean libbalsa_is_sending_mail(void);
+void libbalsa_wait_for_sending_thread(gint max_seconds);
 
 void libbalsa_lock_mutt(void);
 void libbalsa_unlock_mutt(void);
