@@ -1,7 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -54,7 +53,24 @@ struct _LibBalsaMailboxImapClass {
 
 GtkObject *libbalsa_mailbox_imap_new(void);
 
+void libbalsa_mailbox_imap_update_url(LibBalsaMailboxImap* mailbox);
 void libbalsa_mailbox_imap_set_path(LibBalsaMailboxImap * mailbox,
-				    gchar * path);
+				    const gchar * path);
+
+void libbalsa_mailbox_imap_subscribe(LibBalsaMailboxImap * mailbox, 
+				     gboolean subscribe);
+
+void libbalsa_imap_close_all_connections(void);
+
+void libbalsa_imap_new_subfolder(const gchar * parent, const gchar * folder,
+				 gboolean subscribe, LibBalsaServer * server);
+
+void libbalsa_imap_rename_subfolder(const gchar *dir, const gchar *parent,
+				    const gchar *folder, gboolean subscribe, 
+				    LibBalsaServer *server);
+
+void libbalsa_imap_delete_folder(LibBalsaMailboxImap * mailbox);
+
+void reset_mutt_passwords(LibBalsaServer *);
 
 #endif				/* __LIBBALSA_MAILBOX_IMAP_H__ */
