@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -39,20 +39,22 @@
 #include "threads.h"
 #endif
 
+enum {
+    REMOVE_FILES,
+    LAST_SIGNAL
+};
+static guint libbalsa_mailbox_local_signals[LAST_SIGNAL];
+
 static LibBalsaMailboxClass *parent_class = NULL;
 
-static void libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass *
-					      klass);
+static void libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass *klass);
 static void libbalsa_mailbox_local_init(LibBalsaMailboxLocal * mailbox);
 static void libbalsa_mailbox_local_destroy(GtkObject * object);
 
-static void libbalsa_mailbox_local_open(LibBalsaMailbox * mailbox,
-					gboolean append);
+static void libbalsa_mailbox_local_open(LibBalsaMailbox * mailbox);
+static LibBalsaMailboxAppendHandle*
+libbalsa_mailbox_local_append(LibBalsaMailbox * mailbox);
 static void libbalsa_mailbox_local_check(LibBalsaMailbox * mailbox);
-static FILE *libbalsa_mailbox_local_get_message_stream(LibBalsaMailbox *
-						       mailbox,
-						       LibBalsaMessage *
-						       message);
 
 static void libbalsa_mailbox_local_save_config(LibBalsaMailbox * mailbox,
 					       const gchar * prefix);
@@ -93,31 +95,42 @@ libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass * klass)
 
     parent_class = gtk_type_class(libbalsa_mailbox_get_type());
 
+    libbalsa_mailbox_local_signals[REMOVE_FILES] =
+	gtk_signal_new("remove-files",
+		       GTK_RUN_LAST,
+		       object_class->type,
+		       GTK_SIGNAL_OFFSET(LibBalsaMailboxLocalClass,
+					 remove_files),
+		       gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
+
+    gtk_object_class_add_signals(object_class, libbalsa_mailbox_local_signals,
+				 LAST_SIGNAL);
+
+
     object_class->destroy = libbalsa_mailbox_local_destroy;
 
     libbalsa_mailbox_class->open_mailbox = libbalsa_mailbox_local_open;
-    libbalsa_mailbox_class->get_message_stream =
-	libbalsa_mailbox_local_get_message_stream;
+    libbalsa_mailbox_class->open_mailbox_append 
+	= libbalsa_mailbox_local_append;
     libbalsa_mailbox_class->check = libbalsa_mailbox_local_check;
 
     libbalsa_mailbox_class->save_config =
 	libbalsa_mailbox_local_save_config;
     libbalsa_mailbox_class->load_config =
 	libbalsa_mailbox_local_load_config;
+
+    klass->remove_files = NULL;
 }
 
 static void
 libbalsa_mailbox_local_init(LibBalsaMailboxLocal * mailbox)
 {
-    mailbox->path = NULL;
 }
 
 GtkObject *
 libbalsa_mailbox_local_new(const gchar * path, gboolean create)
 {
-    LibBalsaMailbox *mailbox;
-    LibBalsaMailboxType type;
-    int fd, magic_type;
+    int magic_type;
 
     libbalsa_lock_mutt();
     magic_type = mx_get_magic(path);
@@ -126,181 +139,173 @@ libbalsa_mailbox_local_new(const gchar * path, gboolean create)
     switch (magic_type) {
     case M_MBOX:
     case M_MMDF:
-	type = LIBBALSA_MAILBOX_LOCAL_MBOX;
+	return libbalsa_mailbox_mbox_new(path, create);
 	break;
     case M_MH:
-	type = LIBBALSA_MAILBOX_LOCAL_MH;
+	return libbalsa_mailbox_mh_new(path, create);
 	break;
     case M_MAILDIR:
-	type = LIBBALSA_MAILBOX_LOCAL_MAILDIR;
+	return libbalsa_mailbox_maildir_new(path, create);
 	break;
     case M_IMAP:
 	g_warning("Got IMAP as type for local mailbox\n");
 	return NULL;
     default:			/* mailbox non-existent or unreadable */
-	if (!create)
-	    return NULL;
-	if ((fd = creat(path, S_IRUSR | S_IWUSR)) == -1) {
-	    g_warning("An error:\n%s\n occured while trying to "
-		      "create the mailbox \"%s\"\n",
-		      strerror(errno), path);
-	    return NULL;
-	} else
-	    close(fd);
+	if ( create ) 
+	    return libbalsa_mailbox_mbox_new(path, TRUE);
+	g_warning("Unknown mailbox type\n");
+	return NULL;
     }
-
-    mailbox = gtk_type_new(LIBBALSA_TYPE_MAILBOX_LOCAL);
-
-    if (type != LIBBALSA_MAILBOX_LOCAL_MBOX)
-	mailbox->is_directory = TRUE;
-
-    LIBBALSA_MAILBOX_LOCAL(mailbox)->type = type;
-    LIBBALSA_MAILBOX_LOCAL(mailbox)->path = g_strdup(path);
-
-    libbalsa_notify_register_mailbox(mailbox);
-
-    return GTK_OBJECT(mailbox);
 }
 
 /* libbalsa_mailbox_local_set_path:
    returrns errno on error, 0 on success
+   FIXME: proper suport for maildir and mh
 */
 gint
 libbalsa_mailbox_local_set_path(LibBalsaMailboxLocal * mailbox,
 				const gchar * path)
 {
-    int i;
-    /* rename */
-    g_return_val_if_fail(path, 0);
-    if (strcmp(path, LIBBALSA_MAILBOX_LOCAL(mailbox)->path) == 0)
-	return 0;
-    if ((i = rename(LIBBALSA_MAILBOX_LOCAL(mailbox)->path, path)) != 0)
-	return i;
+    int i = 0;
+
+    g_return_val_if_fail(mailbox, -1);
+    g_return_val_if_fail(path, -1);
+    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox), -1);
+
+    if ( LIBBALSA_MAILBOX(mailbox)->url != NULL ) {
+	const gchar* cur_path = libbalsa_mailbox_local_get_path(mailbox);
+	if (g_strcasecmp(path, cur_path) == 0)
+	    return 0;
+	else 
+	    i = rename(cur_path, path);
+    } else {
+	if(LIBBALSA_IS_MAILBOX_MAILDIR(mailbox))
+	    i = libbalsa_mailbox_maildir_create(path, TRUE);
+	else if(LIBBALSA_IS_MAILBOX_MH(mailbox))
+	    i = libbalsa_mailbox_mh_create(path, TRUE);
+	else if(LIBBALSA_IS_MAILBOX_MBOX(mailbox))
+	    i = libbalsa_mailbox_mbox_create(path, TRUE);	    
+    }
 
     /* update mailbox data */
-    libbalsa_notify_unregister_mailbox(LIBBALSA_MAILBOX(mailbox));
-    g_free(mailbox->path);
-    mailbox->path = g_strdup(path);
-    libbalsa_notify_register_mailbox(LIBBALSA_MAILBOX(mailbox));
-    return 0;
+    if(!i) {
+	libbalsa_notify_unregister_mailbox(LIBBALSA_MAILBOX(mailbox));
+	g_free(LIBBALSA_MAILBOX(mailbox)->url);
+	LIBBALSA_MAILBOX(mailbox)->url = g_strconcat("file://", path, NULL);
+	libbalsa_notify_register_mailbox(LIBBALSA_MAILBOX(mailbox));
+	return 0;
+    } else
+	return errno;
+}
+
+void
+libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal *mailbox)
+{
+    g_return_if_fail (LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
+
+    gtk_signal_emit(GTK_OBJECT(mailbox),
+		    libbalsa_mailbox_local_signals[REMOVE_FILES]);
+
 }
 
 static void
 libbalsa_mailbox_local_destroy(GtkObject * object)
 {
-    LibBalsaMailboxLocal *mailbox;
+    LibBalsaMailbox *mailbox;
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(object));
 
-    mailbox = LIBBALSA_MAILBOX_LOCAL(object);
-
-    libbalsa_notify_unregister_mailbox(LIBBALSA_MAILBOX(mailbox));
-
-    g_free(mailbox->path);
+    mailbox = LIBBALSA_MAILBOX(object);
+    libbalsa_notify_unregister_mailbox(mailbox);
 
     if (GTK_OBJECT_CLASS(parent_class)->destroy)
 	(*GTK_OBJECT_CLASS(parent_class)->destroy) (GTK_OBJECT(object));
 }
 
+/* libbalsa_mailbox_local_open:
+   THREADING: it is always called from a signal handler so the gdk lock
+   is held on entry. The lock is released on the time-consuming open
+   operation and taken back when job is finished.
+   Order is crucial to avoid deadlocks.
+*/
 static void
-libbalsa_mailbox_local_open(LibBalsaMailbox * mailbox, gboolean append)
+libbalsa_mailbox_local_open(LibBalsaMailbox * mailbox)
 {
     struct stat st;
     LibBalsaMailboxLocal *local;
+    const gchar* path;
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
 
-    LOCK_MAILBOX(mailbox);
+    gdk_threads_leave();
 
+    LOCK_MAILBOX(mailbox);
     local = LIBBALSA_MAILBOX_LOCAL(mailbox);
+    path = libbalsa_mailbox_local_get_path(mailbox);
 
     if (CLIENT_CONTEXT_OPEN(mailbox)) {
-	if (append) {
-	    /* we need the mailbox to be opened fresh i think */
-	    libbalsa_lock_mutt();
-	    mx_close_mailbox(CLIENT_CONTEXT(mailbox), NULL);
-	    libbalsa_unlock_mutt();
-	} else {
-	    /* incriment the reference count */
-	    mailbox->open_ref++;
-
-	    UNLOCK_MAILBOX(mailbox);
-	    return;
-	}
-    }
-
-    if (stat(local->path, &st) == -1) {
+	/* incriment the reference count */
+	mailbox->open_ref++;
 	UNLOCK_MAILBOX(mailbox);
+	gdk_threads_enter();
 	return;
     }
 
-    libbalsa_lock_mutt();
-    if (append)
-	CLIENT_CONTEXT(mailbox) =
-	    mx_open_mailbox(local->path, M_APPEND, NULL);
-    else
-	CLIENT_CONTEXT(mailbox) = mx_open_mailbox(local->path, 0, NULL);
-    libbalsa_unlock_mutt();
-
-    if (CLIENT_CONTEXT_OPEN(mailbox)) {
-	mailbox->readonly = CLIENT_CONTEXT(mailbox)->readonly;
-	mailbox->messages = 0;
-	mailbox->total_messages = 0;
-	mailbox->unread_messages = 0;
-	mailbox->new_messages = CLIENT_CONTEXT(mailbox)->msgcount;
-	libbalsa_mailbox_load_messages(mailbox);
-
-	/* increment the reference count */
-	mailbox->open_ref++;
-
-#ifdef DEBUG
-	g_print(_("LibBalsaMailboxLocal: Opening %s Refcount: %d\n"),
-		mailbox->name, mailbox->open_ref);
-#endif
+    if (stat(path, &st) == -1) {
+	UNLOCK_MAILBOX(mailbox);
+	gdk_threads_enter();
+	return;
     }
-
+    
+    libbalsa_lock_mutt();
+    CLIENT_CONTEXT(mailbox) = mx_open_mailbox(path, 0, NULL);
+    libbalsa_unlock_mutt();
+    
+    if (!CLIENT_CONTEXT_OPEN(mailbox)) {
+	UNLOCK_MAILBOX(mailbox);
+	gdk_threads_enter();
+	return;
+    }
+    mailbox->readonly = CLIENT_CONTEXT(mailbox)->readonly;
+    mailbox->messages = 0;
+    mailbox->total_messages = 0;
+    mailbox->unread_messages = 0;
+    mailbox->new_messages = CLIENT_CONTEXT(mailbox)->msgcount
+	- CLIENT_CONTEXT(mailbox)->deleted;
+    mailbox->open_ref++;
     UNLOCK_MAILBOX(mailbox);
-
+    gdk_threads_enter();
+    libbalsa_mailbox_load_messages(mailbox);
+    
+    /* increment the reference count */
+#ifdef DEBUG
+    g_print(_("LibBalsaMailboxLocal: Opening %s Refcount: %d\n"),
+	    mailbox->name, mailbox->open_ref);
+#endif
 }
 
-static FILE *
-libbalsa_mailbox_local_get_message_stream(LibBalsaMailbox * mailbox,
-					  LibBalsaMessage * message)
+static LibBalsaMailboxAppendHandle*
+libbalsa_mailbox_local_append(LibBalsaMailbox * mailbox)
 {
-    LibBalsaMailboxLocal *local;
+    LibBalsaMailboxAppendHandle* res;
+    g_return_val_if_fail(mailbox, NULL);
 
-    FILE *stream = NULL;
-    gchar *filename;
-
-    g_return_val_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox), NULL);
-    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), NULL);
-
-    local = LIBBALSA_MAILBOX_LOCAL(mailbox);
-
-    switch (local->type) {
-    case LIBBALSA_MAILBOX_LOCAL_MH:
-    case LIBBALSA_MAILBOX_LOCAL_MAILDIR:
-	filename =
-	    g_strdup_printf("%s/%s", local->path,
-			    libbalsa_message_pathname(message));
-	stream = fopen(filename, "r");
-	if (!stream || ferror(stream)) {
-	    fprintf(stderr, "Open of %s failed. Errno = %d, ",
-		    filename, errno);
-	    g_free(filename);
-	    perror(NULL);
-	    return NULL;
-	}
-	break;
-    case LIBBALSA_MAILBOX_LOCAL_MBOX:
-	stream = fopen(local->path, "r");
-	break;
-    default:
-	g_error("Unknown type of local mailbox %d\n", local->type);
-	break;
+    res = g_new0(LibBalsaMailboxAppendHandle,1);
+    libbalsa_lock_mutt();
+    res->context = mx_open_mailbox(libbalsa_mailbox_local_get_path(mailbox), 
+				   M_APPEND, NULL);
+    if(res->context == NULL) {
+	g_free(res);
+	res = NULL;
+    } else if (res->context->readonly) {
+	g_warning("Cannot open dest local mailbox '%s' for writing.", 
+		  mailbox->name);
+	mx_close_mailbox(res->context, NULL);
+	g_free(res);
+	res = NULL;
     }
-
-    return stream;
+    libbalsa_unlock_mutt();
+    return res;
 }
 
 static void
@@ -323,12 +328,15 @@ libbalsa_mailbox_local_check(LibBalsaMailbox * mailbox)
 
 	if (i < 0) {
 	    g_print("mx_check_mailbox() failed on %s\n", mailbox->name);
-	} else if (i == M_NEW_MAIL || i == M_REOPENED) {
+	} 
+	if (i == M_NEW_MAIL || i == M_REOPENED) {
 	    mailbox->new_messages =
 		CLIENT_CONTEXT(mailbox)->msgcount - mailbox->messages;
+	    UNLOCK_MAILBOX(mailbox);
 	    libbalsa_mailbox_load_messages(mailbox);
+	} else {
+	    UNLOCK_MAILBOX(mailbox);
 	}
-	UNLOCK_MAILBOX(mailbox);
     }
 }
 
@@ -342,7 +350,7 @@ libbalsa_mailbox_local_save_config(LibBalsaMailbox * mailbox,
 
     local = LIBBALSA_MAILBOX_LOCAL(mailbox);
 
-    gnome_config_set_string("Path", local->path);
+    gnome_config_set_string("Path", libbalsa_mailbox_local_get_path(local));
 
     if (LIBBALSA_MAILBOX_CLASS(parent_class)->save_config)
 	LIBBALSA_MAILBOX_CLASS(parent_class)->save_config(mailbox, prefix);
@@ -353,36 +361,16 @@ libbalsa_mailbox_local_load_config(LibBalsaMailbox * mailbox,
 				   const gchar * prefix)
 {
     LibBalsaMailboxLocal *local;
-
+    gchar* path;
     g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
 
     local = LIBBALSA_MAILBOX_LOCAL(mailbox);
 
-    g_free(local->path);
-
-    local->path = gnome_config_get_string("Path");
-
-    /* FIXME: Maybe we should save the type... */
-
-    libbalsa_lock_mutt();
-
-    switch (mx_get_magic(local->path)) {
-    case M_MBOX:
-    case M_MMDF:
-	local->type = LIBBALSA_MAILBOX_LOCAL_MBOX;
-	break;
-    case M_MH:
-	local->type = LIBBALSA_MAILBOX_LOCAL_MH;
-	break;
-    case M_MAILDIR:
-	local->type = LIBBALSA_MAILBOX_LOCAL_MAILDIR;
-	break;
-    default:
-	local->type = LIBBALSA_MAILBOX_LOCAL_MBOX;
-    }
-
-    libbalsa_unlock_mutt();
+    g_free(mailbox->url);
 
+    path = gnome_config_get_string("Path");
+    mailbox->url = g_strconcat("file://", path, NULL);
+    g_free(path);
 
     if (LIBBALSA_MAILBOX_CLASS(parent_class)->load_config)
 	LIBBALSA_MAILBOX_CLASS(parent_class)->load_config(mailbox, prefix);
