@@ -53,7 +53,7 @@ pop_get_errstr(PopStatus status)
     static gchar* errmsgs[] = {
 	"",
 	N_("connection error"),
-	N_("POP command exection failed"),
+	N_("POP command execution failed"),
 	N_("Could not write the message"),
 	N_("Could not run the delivery program (procmail)"),
 	N_("Could not open mailbox for spooling"),
@@ -137,20 +137,22 @@ getLine (int fd, char *s, int len)
 */
 
 static gboolean
-getApopStamp (char *buff, char *stamp) {
+getApopStamp(char *buff, char *stamp)
+{
     char *start;
     char *finish;
-    size_t len;
-    
+
+    g_return_val_if_fail(buff, FALSE);
+    g_return_val_if_fail(stamp, FALSE);
     start = strchr(buff, '<');
-    finish = strchr(buff, '>');
-    
-    if( buff && stamp && start && finish ) {
-        len = strlen(start) - strlen(finish) + 1;
-        strncpy( stamp, start, len );
-        return TRUE;
-    }
-    else return FALSE;
+    if (start) {
+        finish = strchr(start, '>');
+        if (finish) {
+            strncpy(stamp, start, finish - start + 1);
+            return TRUE;
+        }
+    } 
+    return FALSE;
 }
 
 /* Compute the authentication hash to send to the server - kabir */
@@ -591,10 +593,11 @@ fetch_pop_mail (const gchar *pop_host, const gchar *pop_user,
 
     DM("POP3 - begin connection");
     status = pop_connect(&s, pop_host, pop_port);
+    if(status != POP_OK) return status;
     DM("POP3 Connected; hello");
 
-    if(status == POP_OK)
-	status = pop_auth(s, pop_user, pop_pass, use_apop);
+    
+    status = pop_auth(s, pop_user, pop_pass, use_apop);
 
     DM("POP3 authentication %s successful", (status ==POP_OK ? "" : "NOT"));
     if(status == POP_OK)
