@@ -245,6 +245,10 @@ enum
   OPT_VERIFYSIG, /* verify PGP signatures */
 #endif
 
+#ifdef USE_SSL
+  OPT_SSLSTARTTLS,
+#endif
+
   OPT_PRINT,
   OPT_INCLUDE,
   OPT_DELETE,
@@ -307,6 +311,7 @@ enum
 #ifdef USE_IMAP
   OPTIMAPLSUB,
   OPTIMAPPASSIVE,
+  OPTIMAPPEEK,
   OPTIMAPSERVERNOISE,
 # ifdef USE_SSL
   OPTIMAPFORCESSL,
@@ -351,12 +356,16 @@ enum
   OPTSTATUSONTOP,
   OPTSTRICTTHREADS,
   OPTSUSPEND,
+  OPTTEXTFLOWED,
   OPTTHOROUGHSRC,
   OPTTILDE,
   OPTUNCOLLAPSEJUMP,
   OPTUSE8BITMIME,
   OPTUSEDOMAIN,
   OPTUSEFROM,
+#ifdef HAVE_GETADDRINFO
+  OPTUSEIPV6,
+#endif
   OPTWAITKEY,
   OPTWEED,
   OPTWRAP,
@@ -402,7 +411,10 @@ enum
   OPTNEEDRESCORE,	/* (pseudo) set when the `score' command is used */
   OPTUSEHEADERDATE,	/* (pseudo) used by edit-message */
   OPTATTACHMSG,		/* (pseudo) used by attach-message */
-  
+  OPTKEEPQUIET,         /* (pseudo) shut up the message and refresh
+                         *          functions while we are executing an
+                         *          external program.
+                         */  
 #ifdef _PGPPATH
   OPTPGPCHECKTRUST,	/* (pseudo) used by pgp_select_key () */
   OPTDONTHANDLEPGPKEYS,	/* (pseudo) used to extract PGP keys */
@@ -462,12 +474,15 @@ typedef struct envelope
   ADDRESS *bcc;
   ADDRESS *sender;
   ADDRESS *reply_to;
+ /* BALSA: added dispnotify_to field */
+  ADDRESS *dispnotify_to;
   ADDRESS *mail_followup_to;
   char *subject;
   char *real_subj;		/* offset of the real subject */
   char *message_id;
   char *supersedes;
   char *date;
+  char *x_label;
   LIST *references;		/* message references (in reverse order) */
   LIST *userhdrs;		/* user defined headers */
 } ENVELOPE;
@@ -539,6 +554,15 @@ typedef struct body
   unsigned int tagged : 1;
   unsigned int deleted : 1;	/* attachment marked for deletion */
 
+  unsigned int noconv : 1;      /* don't do character set conversion */
+  unsigned int force_charset : 1; 
+                                /* send mode: don't adjust the character
+                                 * set when in send-mode.
+                                 */
+
+  unsigned int collapsed : 1;   /* used by recvattach */
+
+
 } BODY;
 
 typedef struct header
@@ -629,7 +653,9 @@ typedef struct pattern_t
   regex_t *rx;
 } pattern_t;
 
-typedef struct
+/* BALSA: make the structure non-anonymous so we can have pointers to it
+   without parsing this file for the struct definition. */
+struct _CONTEXT
 {
   char *path;
   FILE *fp;
@@ -668,7 +694,8 @@ typedef struct
   unsigned int revsort : 1;	/* mailbox sorted in reverse? */
   unsigned int collapsed : 1;   /* are all threads collapsed? */
   unsigned int closing : 1;	/* mailbox is being closed */
-} CONTEXT;
+};
+typedef struct _CONTEXT CONTEXT;
 
 typedef struct attachptr
 {
@@ -711,5 +738,6 @@ void state_prefix_putc(char, STATE *);
 int  state_printf(STATE *, const char *, ...);
 
 #include "protos.h"
+#include "lib.h"
 #include "globals.h"
 #endif
