@@ -460,7 +460,9 @@ int mutt_write_mime_header (BODY *a, FILE *f)
       
       buffer[0] = 0;
       rfc822_cat (buffer, sizeof (buffer), t, MimeSpecials);
-      fprintf (f, "; filename=%s", buffer);
+      /* BALSA: don't include filename for `inline' disposition (see RFC2183) */
+      if (a->disposition != DISPINLINE)
+	fprintf (f, "; filename=%s", buffer);
     }
 
     fputc ('\n', f);
@@ -554,11 +556,27 @@ int mutt_write_mime_body (BODY *a, FILE *f)
 void mutt_generate_boundary (PARAMETER **parm)
 {
   char rs[BOUNDARYLEN + 1];
-  char *p = rs;
+  char *p;
   int i;
+#ifdef LIBMUTT
+  static int seeded = 0;
+
+  if (!seeded) {
+    SRAND(time(NULL));
+    ++seeded;
+  }
+#endif
 
   rs[BOUNDARYLEN] = 0;
-  for (i=0;i<BOUNDARYLEN;i++) 
+  /* start the boundary with `=_', as suggested in RFC 2045:
+   * ensures that it can't match a string in a quoted-printable part;
+   * also ensures that it can't match a string in a base64-encoded 
+   * part, which would also seem to be a possibility with this
+   * construction. */
+  rs[0] = '=';
+  rs[1] = '_';
+  p = &rs[2];
+  for (i=2;i<BOUNDARYLEN;i++) 
     *p++ = B64Chars[LRAND() % sizeof (B64Chars)];
   *p = 0;
   
