@@ -46,6 +46,9 @@ struct _AddressBookConfig {
 	struct {
 	    GtkWidget *path;
 	} vcard;
+	struct {
+	    GtkWidget *path;
+	} ldif;
 #ifdef ENABLE_LDAP
 	struct {
 	    GtkWidget *host_name;
@@ -59,6 +62,7 @@ struct _AddressBookConfig {
 
 static GtkWidget *create_choice_page(AddressBookConfig * abc);
 static GtkWidget *create_vcard_page(AddressBookConfig * abc);
+static GtkWidget *create_ldif_page(AddressBookConfig * abc);
 #ifdef ENABLE_LDAP
 static GtkWidget *create_ldap_page(AddressBookConfig * abc);
 #endif
@@ -101,6 +105,7 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
     gnome_dialog_set_parent(GNOME_DIALOG(abc->window),
 			    GTK_WINDOW(balsa_app.main_window));
     gnome_dialog_close_hides(GNOME_DIALOG(abc->window), TRUE);
+    gtk_window_set_wmclass(GTK_WINDOW(abc->window), "address_book_config_dialog", "Balsa");
 
     abc->notebook = gtk_notebook_new();
     gtk_container_set_border_width(GTK_CONTAINER(abc->window), 5);
@@ -141,12 +146,16 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 
 	if (LIBBALSA_IS_ADDRESS_BOOK_VCARD(address_book)) {
 	    page = create_vcard_page(abc);
+	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book)) {
+	    page = create_ldif_page(abc);
 #ifdef ENABLE_LDAP
 	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDAP(address_book)) {
 	    page = create_ldap_page(abc);
 #endif
-	} else
+	} else {
 	    g_assert_not_reached();
+	    page = NULL;
+	}
     }
 
     gtk_notebook_append_page(GTK_NOTEBOOK(abc->notebook), page, NULL);
@@ -180,6 +189,14 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 					       (abc->ab_specific.vcard.path), FALSE);
 	    if (path != NULL) 
 		address_book = libbalsa_address_book_vcard_new(name, path);
+            g_free(path);
+	} else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
+	    gchar *path =
+		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
+					       (abc->ab_specific.ldif.path), FALSE);
+	    if (path != NULL)
+		address_book = libbalsa_address_book_ldif_new(name, path);
+            g_free(path);
 #ifdef ENABLE_LDAP
 	} else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDAP) {
 	    gchar *host_name =
@@ -211,7 +228,19 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 	    vcard = LIBBALSA_ADDRESS_BOOK_VCARD(address_book);
 	    if (path) {
 		g_free(vcard->path);
-		vcard->path = g_strdup(path);
+		vcard->path = path;
+	    }
+	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book)) {
+	    LibBalsaAddressBookLdif *ldif;
+	    gchar *path =
+		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
+					       (abc->ab_specific.ldif.path), 
+					       FALSE);
+
+	    ldif = LIBBALSA_ADDRESS_BOOK_LDIF(address_book);
+	    if (path) {
+		g_free(ldif->path);
+		ldif->path = path;
 	    }
 #ifdef ENABLE_LDAP
 	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDAP(address_book)) {
@@ -273,6 +302,18 @@ create_choice_page(AddressBookConfig * abc)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button), TRUE);
     abc->create_type = LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
 
+    /* ... LDIF Address Book */
+    radio_button = gtk_radio_button_new_with_label
+	(gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button)),
+	 _("LDIF Address Book"));
+    gtk_box_pack_start(GTK_BOX(vbox), radio_button, FALSE, FALSE, 0);
+    gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
+		       GTK_SIGNAL_FUNC(set_the_page), (gpointer) abc);
+    gtk_object_set_user_data(GTK_OBJECT(radio_button),
+			     GINT_TO_POINTER
+			     (LIBBALSA_TYPE_ADDRESS_BOOK_LDIF));
+    gtk_widget_show(radio_button);
+
     /* ... LDAP. */
     /* The dialog will look weird if LDAP isn't enabled
      * since there will be only one option...
@@ -303,20 +344,20 @@ static GtkWidget *
 create_vcard_page(AddressBookConfig * abc)
 {
     GtkWidget *table;
+    GnomeDialog* mcw = GNOME_DIALOG(abc->window);
     GtkWidget *label;
+    guint keyval;
+    LibBalsaAddressBookVcard* ab;
+    ab = (LibBalsaAddressBookVcard*)abc->address_book; /* may be NULL */
 
     table = gtk_table_new(2, 3, FALSE);
 
     /* mailbox name */
 
-    label = gtk_label_new(_("Address Book Name:"));
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-		     GTK_FILL, GTK_FILL, 10, 10);
-
-    abc->name_entry = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table), abc->name_entry, 1, 2, 0, 1,
-		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
+    create_label(_("_Address Book Name"), table, 0, &keyval);
+    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+				   ab ? abc->address_book->name : NULL, 
+				   keyval);
 
     label = gtk_label_new(_("File name"));
     gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -325,30 +366,71 @@ create_vcard_page(AddressBookConfig * abc)
 
     abc->ab_specific.vcard.path =
 	gnome_file_entry_new("VCARD ADDRESS BOOK PATH",
-			     "Select path for address book");
+			     _("Select path for address book"));
     gtk_table_attach(GTK_TABLE(table), abc->ab_specific.vcard.path, 1, 2,
 		     1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
 
     abc->expand_aliases_button =
-	gtk_check_button_new_with_label(_("Expand aliases as you type"));
-    gtk_table_attach(GTK_TABLE(table), abc->expand_aliases_button, 1, 2, 2,
-		     3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
+	create_check(mcw, _("_Expand aliases as you type"), table, 3,
+		     ab ? abc->address_book->expand_aliases : TRUE);
+
+    if (ab) {
+	GtkWidget *entry;
+	entry = GTK_WIDGET(gnome_file_entry_gtk_entry
+			   (GNOME_FILE_ENTRY(abc->ab_specific.vcard.path)));
+	gtk_entry_set_text(GTK_ENTRY(entry), ab->path);
+    }
+    gtk_widget_show_all(table);
+    return table;
+
+}
+
+static GtkWidget *
+create_ldif_page(AddressBookConfig * abc)
+{
+    GtkWidget *table;
+    GtkWidget *label;
+    int keyval;
+    GnomeDialog* mcw = GNOME_DIALOG(abc->window);
+
+    table = gtk_table_new(2, 3, FALSE);
+
+    /* mailbox name */
+
+    create_label(_("_Address Book Name"), table, 0, &keyval);
+    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+				   abc->address_book 
+				   ? abc->address_book->name : NULL, 
+				   keyval);
+
+    label = gtk_label_new(_("File name"));
+    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
+		     GTK_FILL, GTK_FILL, 10, 10);
+
+    abc->ab_specific.ldif.path =
+	gnome_file_entry_new("LDIF ADDRESS BOOK PATH",
+			     _("Select path for address book"));
+    gtk_table_attach(GTK_TABLE(table), abc->ab_specific.ldif.path, 1, 2,
+		     1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
+
+    abc->expand_aliases_button =
+	create_check(mcw, _("_Expand aliases as you type"), table, 2,
+		     abc->address_book ? 
+		     abc->address_book->expand_aliases : TRUE);
 
     if (abc->address_book) {
-	LibBalsaAddressBookVcard *vcard;
+	LibBalsaAddressBookLdif *ldif;
 	GtkWidget *entry;
 
-	vcard = LIBBALSA_ADDRESS_BOOK_VCARD(abc->address_book);
+	ldif = LIBBALSA_ADDRESS_BOOK_LDIF(abc->address_book);
 	entry =
 	    GTK_WIDGET(gnome_file_entry_gtk_entry
-		       (GNOME_FILE_ENTRY(abc->ab_specific.vcard.path)));
+		       (GNOME_FILE_ENTRY(abc->ab_specific.ldif.path)));
 
 	gtk_entry_set_text(GTK_ENTRY(abc->name_entry),
 			   abc->address_book->name);
-	gtk_entry_set_text(GTK_ENTRY(entry), vcard->path);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
-				     (abc->expand_aliases_button),
-				     abc->address_book->expand_aliases);
+	gtk_entry_set_text(GTK_ENTRY(entry), ldif->path);
     }
 
     gtk_widget_show_all(table);
@@ -360,65 +442,34 @@ create_vcard_page(AddressBookConfig * abc)
 static GtkWidget *
 create_ldap_page(AddressBookConfig * abc)
 {
-    GtkWidget *table;
-    GtkWidget *label;
+    GtkWidget *table = gtk_table_new(2, 3, FALSE);
 
-    table = gtk_table_new(2, 3, FALSE);
+    LibBalsaAddressBookLdap* ab;
+    GnomeDialog* mcw = GNOME_DIALOG(abc->window);
+    guint keyval;
+
+    ab = (LibBalsaAddressBookLdap*)abc->address_book; /* may be NULL */
 
     /* mailbox name */
 
-    label = gtk_label_new(_("Address Book Name:"));
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-		     GTK_FILL, GTK_FILL, 10, 10);
-    gtk_widget_show(label);
+    create_label(_("_Address Book Name"), table, 0, &keyval);
+    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+				   ab ? abc->address_book->name : NULL, 
+				   keyval);
 
-    abc->name_entry = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table), abc->name_entry, 1, 2, 0, 1,
-		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
-    gtk_widget_show(abc->name_entry);
+    create_label(_("_Host Name"), table, 1, &keyval);
+    abc->ab_specific.ldap.host_name = 
+	create_entry(mcw, table, NULL, NULL, 1, 
+		     ab ? ab->host : NULL, keyval);
 
-    label = gtk_label_new(_("Host name"));
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-		     GTK_FILL, GTK_FILL, 10, 10);
-    gtk_widget_show(label);
-
-    abc->ab_specific.ldap.host_name = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table), abc->ab_specific.ldap.host_name, 1,
-		     2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
-    gtk_widget_show(abc->ab_specific.ldap.host_name);
-
-    label = gtk_label_new(_("Base Domain Name"));
-    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-		     GTK_FILL, GTK_FILL, 10, 10);
-    gtk_widget_show(label);
-
-    abc->ab_specific.ldap.base_dn = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table), abc->ab_specific.ldap.base_dn, 1, 2,
-		     2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
-    gtk_widget_show(abc->ab_specific.ldap.base_dn);
+    create_label(_("_Base Domain Name"), table, 2, &keyval);
+    abc->ab_specific.ldap.base_dn = 
+	create_entry(mcw, table, NULL, NULL, 2, 
+		     ab ? ab->base_dn : NULL, keyval);
 
     abc->expand_aliases_button =
-	gtk_check_button_new_with_label(_("Expand aliases as you type"));
-    gtk_table_attach(GTK_TABLE(table), abc->expand_aliases_button, 1, 2, 3,
-		     4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
-
-    if (abc->address_book) {
-	LibBalsaAddressBookLdap *ldap;
-	ldap = LIBBALSA_ADDRESS_BOOK_LDAP(abc->address_book);
-
-	gtk_entry_set_text(GTK_ENTRY(abc->name_entry),
-			   abc->address_book->name);
-	gtk_entry_set_text(GTK_ENTRY(abc->ab_specific.ldap.host_name),
-			   ldap->host);
-	gtk_entry_set_text(GTK_ENTRY(abc->ab_specific.ldap.base_dn),
-			   ldap->base_dn);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
-				     (abc->expand_aliases_button),
-				     abc->address_book->expand_aliases);
-    }
+	create_check(mcw, _("_Expand aliases as you type"), table, 3,
+		     ab ? abc->address_book->expand_aliases : TRUE);
 
     gtk_widget_show(table);
     return table;
@@ -467,12 +518,15 @@ next_button_cb(GtkWidget * button, AddressBookConfig * abc)
 
     if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD)
 	page = create_vcard_page(abc);
+    else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF)
+	page = create_ldif_page(abc);
 #ifdef ENABLE_LDAP
     else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDAP)
 	page = create_ldap_page(abc);
 #endif
-    else
-	g_assert_not_reached();
+    else {
+	g_assert_not_reached(); page = NULL;
+    }
 
     gtk_notebook_append_page(GTK_NOTEBOOK(abc->notebook), page, NULL);
 
@@ -519,6 +573,32 @@ handle_close(AddressBookConfig * abc)
 	    else abc->cancelled = TRUE;
 	} else g_free(path);
     }
+    else if( (abc->address_book &&
+	 LIBBALSA_IS_ADDRESS_BOOK_LDIF(abc->address_book)) ||
+	(!abc->address_book&&
+	 abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF)) {
+	gchar *path =
+	    gnome_file_entry_get_full_path(
+		GNOME_FILE_ENTRY(abc->ab_specific.ldif.path), FALSE);
+
+	if(!path) {
+	    gchar *msg = g_strdup_printf(
+		_("The address book file path '%s' is not correct.\n"
+		  "Do you want to correct the file name?"),
+		gtk_entry_get_text(GTK_ENTRY(
+		    gnome_file_entry_gtk_entry(
+			GNOME_FILE_ENTRY(abc->ab_specific.ldif.path)))));
+	    ask = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_QUESTION,
+					GNOME_STOCK_BUTTON_OK,
+					GNOME_STOCK_BUTTON_CANCEL, NULL);
+	    g_free(msg);
+	    gnome_dialog_set_default(GNOME_DIALOG(ask), 1);
+	    gnome_dialog_set_parent(GNOME_DIALOG(ask),GTK_WINDOW(abc->window));
+	    clicked_button = gnome_dialog_run_and_close(GNOME_DIALOG(ask));
+	    if(clicked_button == 0) return;
+	    else abc->cancelled = TRUE;
+	} else g_free(path);
+    }
 
     gnome_dialog_close(GNOME_DIALOG(abc->window));
 }
@@ -531,6 +611,5 @@ add_button_cb(GtkWidget * button, AddressBookConfig * abc)
 static void
 update_button_cb(GtkWidget * button, AddressBookConfig * abc)
 {
-    printf("Update buttn cb\n");
     handle_close(abc);
 }
