@@ -29,6 +29,10 @@
 #include "main-window.h"
 #include "information.h"
 
+#ifdef BALSA_USE_THREADS
+#include <pthread.h>
+#endif
+
 /* Global application structure */
 struct BalsaApplication balsa_app;
 
@@ -83,31 +87,72 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
     return passwd;
 }
 
-gboolean
-check_mailbox_password(GNode *nd, gpointer data)
+#ifdef BALSA_USE_THREADS
+typedef struct {
+    pthread_cond_t cond;
+    LibBalsaServer* server;
+    LibBalsaMailbox* mbox;
+    gchar* res;
+} AskPasswdData;
+
+/* ask_passwd_idle:
+   called in MT mode by the main thread.
+ */
+static gboolean
+ask_passwd_idle(gpointer data)
+{
+    AskPasswdData* apd = (AskPasswdData*)data;
+    gdk_threads_enter();
+    apd->res = ask_password_real(apd->server, apd->mbox);
+    gdk_threads_leave();
+    pthread_cond_signal(&apd->cond);
+    return FALSE;
+}
+
+static gchar *
+ask_password_mt(LibBalsaServer * server, LibBalsaMailbox * mbox)
+{
+    static pthread_mutex_t ask_passwd_lock = PTHREAD_MUTEX_INITIALIZER;
+    AskPasswdData apd;
+
+    gdk_threads_leave();
+    pthread_mutex_lock(&ask_passwd_lock);
+    pthread_cond_init(&apd.cond, NULL);
+    apd.server = server;
+    apd.mbox   = mbox;
+    gtk_idle_add(ask_passwd_idle, &apd);
+    pthread_cond_wait(&apd.cond, &ask_passwd_lock);
+    
+    pthread_cond_destroy(&apd.cond);
+    pthread_mutex_unlock(&ask_passwd_lock);
+    pthread_mutex_destroy(&ask_passwd_lock);
+    gdk_threads_enter();
+    return apd.res;
+}
+#endif
+
+static gboolean
+set_passwd_from_matching_server(GNode *nd, gpointer data)
 {
     LibBalsaServer *server;
     LibBalsaServer *master;
     LibBalsaMailbox *mbox;
     BalsaMailboxNode *node;
 
-    /*
-     * What follows is a mixture of g_return_val_if_fail(), and
-     * if() return FALSE;  It uses the if()... return FALSE;
-     * constructs if the error is /expected/.
-     *
-     * I probably overdid it, but this way the code worked first time...
-     */
     g_return_val_if_fail(nd != NULL, FALSE);
     node = (BalsaMailboxNode *)nd->data;
     g_return_val_if_fail(BALSA_IS_MAILBOX_NODE(node), FALSE);
-    mbox = node->mailbox;
-    g_return_val_if_fail(mbox != NULL, FALSE);
-    g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mbox), FALSE);
+    if(node->server)
+        server = node->server;
+    else {
+        mbox = node->mailbox;
+        g_return_val_if_fail(mbox != NULL, FALSE);
+        g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mbox), FALSE);
 
-    if (!LIBBALSA_IS_MAILBOX_REMOTE(mbox)) return FALSE;
-    server = LIBBALSA_MAILBOX_REMOTE_SERVER(mbox);
-    g_return_val_if_fail(server != NULL, FALSE);
+        if (!LIBBALSA_IS_MAILBOX_REMOTE(mbox)) return FALSE;
+        server = LIBBALSA_MAILBOX_REMOTE_SERVER(mbox);
+        g_return_val_if_fail(server != NULL, FALSE);
+    }
     g_return_val_if_fail(server->host != NULL, FALSE);
     g_return_val_if_fail(server->user != NULL, FALSE);
     if (server->passwd == NULL) return FALSE;
@@ -139,14 +184,18 @@ ask_password(LibBalsaServer *server, LibBalsaMailbox *mbox)
     password = NULL;
     if (mbox) {
         g_node_traverse(balsa_app.mailbox_nodes, G_IN_ORDER, G_TRAVERSE_LEAFS,
-		-1, check_mailbox_password, server);
+		-1, set_passwd_from_matching_server, server);
 	if (server->passwd != NULL) {
 	    password = server->passwd;
 	    server->passwd = NULL;
 	}
     }
     if (!password)
+#ifdef BALSA_USE_THREADS
+	return ask_password_mt(server, mbox);
+#else
 	return ask_password_real(server, mbox);
+#endif
     else
 	return password;
 }
@@ -529,6 +578,7 @@ traverse_find_dir(GNode * node, gpointer * d)
 }
 /* balsa_app_find_by_dir:
    looks for a mailbox node with dir equal to path.
+   returns NULL on failure
 */
 GNode*
 balsa_app_find_by_dir(GNode* root, const gchar* path)
@@ -539,7 +589,7 @@ balsa_app_find_by_dir(GNode* root, const gchar* path)
     d[1] = NULL;
     g_node_traverse(root, G_LEVEL_ORDER, G_TRAVERSE_ALL, -1,
 		    (GNodeTraverseFunc) traverse_find_dir, d);
-    return d[1] ? d[1] : root;
+    return d[1];
 }
 
 
@@ -565,14 +615,12 @@ destroy_mailbox_node(GNode* node, GNode* root)
 static void 
 destroy_mailbox_tree(GNode* node, GNode* root)
 { 
-    g_node_traverse(node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
-		    (GNodeTraverseFunc)destroy_mailbox_node, root);
 }
 
 void
 balsa_remove_children_mailbox_nodes(GNode* gnode)
 {
-    GNode* walk;
+    GNode* walk, *next_sibling;
     g_return_if_fail(gnode);
 
     if(balsa_app.debug)
@@ -580,14 +628,22 @@ balsa_remove_children_mailbox_nodes(GNode* gnode)
 	       gnode->data, BALSA_MAILBOX_NODE(gnode->data)->name
 	       ? BALSA_MAILBOX_NODE(gnode->data)->name : "");
     gtk_clist_freeze(GTK_CLIST(balsa_app.mblist));
-    g_node_children_foreach(gnode, G_TRAVERSE_ALL,
-                    (GNodeForeachFunc)destroy_mailbox_tree, gnode);
-    gtk_clist_thaw(GTK_CLIST(balsa_app.mblist));
-
-    while( (walk = gnode->children) ) {
+    for(walk = g_node_first_child(gnode); walk; walk = next_sibling) {
+        BalsaMailboxNode *mbnode = BALSA_MAILBOX_NODE(walk->data);
+        next_sibling = g_node_next_sibling(walk);
+        if(mbnode==NULL) continue;
+        if(mbnode->parent == NULL) {
+            printf("sparing %s %s\n", 
+                   mbnode->mailbox ? "mailbox" : "folder ",
+                   mbnode->mailbox ? mbnode->mailbox->name : mbnode->name);
+            continue;
+        }
+        g_node_traverse(walk, G_IN_ORDER, G_TRAVERSE_ALL, -1,
+                        (GNodeTraverseFunc)destroy_mailbox_node, gnode);
 	g_node_unlink(walk);
 	g_node_destroy(walk);
     }
+    gtk_clist_thaw(GTK_CLIST(balsa_app.mblist));
 }
 
 /* create_label:
