@@ -79,6 +79,7 @@ extern "C" {
     gboolean balsa_index_load_mailbox_node(BalsaIndex * bindex,
                                            BalsaMailboxNode * mbnode);
     void balsa_index_refresh(BalsaIndex * bindex);
+    void balsa_index_update_tree(BalsaIndex *bindex, gboolean expand);
     void balsa_index_set_threading_type(BalsaIndex * bindex, int thtype);
     void balsa_index_set_sort_order(BalsaIndex * bindex, int column, 
 				    GtkSortType order);
@@ -93,9 +94,10 @@ extern "C" {
     void balsa_index_redraw_current(BalsaIndex *);
 
 /* select up/down the index */
+    void balsa_index_select_next_threaded(BalsaIndex * bindex);
     void balsa_index_select_next(BalsaIndex *);
-    void balsa_index_select_first_unread(BalsaIndex*);
     void balsa_index_select_next_unread(BalsaIndex * bindex);
+    void balsa_index_select_next_flagged(BalsaIndex * bindex);
     void balsa_index_select_previous(BalsaIndex *);
     void balsa_index_select_row(BalsaIndex * bindex, gint row);
 
@@ -110,8 +112,11 @@ extern "C" {
     void balsa_message_replytoall(GtkWidget * widget, gpointer user_data);
     void balsa_message_replytogroup(GtkWidget * widget, gpointer user_data);
 
-    void balsa_message_forward(GtkWidget * widget, gpointer user_data);
-    void balsa_message_continue(GtkWidget * widget, gpointer user_data);
+    void balsa_message_forward_attached(GtkWidget * widget, gpointer data);
+    void balsa_message_forward_inline(GtkWidget * widget, gpointer data);
+    void balsa_message_forward_quoted(GtkWidget * widget, gpointer data);
+    void balsa_message_forward_default(GtkWidget * widget, gpointer data);
+    void balsa_message_continue(GtkWidget * widget, gpointer data);
 
     void balsa_message_move_to_trash(GtkWidget * widget, gpointer user_data);
     void balsa_message_delete(GtkWidget * widget, gpointer user_data);
@@ -132,7 +137,6 @@ extern "C" {
 				   gint, gpointer);
     void balsa_index_refresh_size (GtkNotebook *, GtkNotebookPage *,
 				   gint, gpointer);
-    gint balsa_index_get_largest_selected(GtkCList *clist);
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
