@@ -703,18 +703,18 @@ mailbox_nodes_to_ctree(GtkCTree * ctree, guint depth, GNode * gnode,
 	else {
 	    BalsaIconName in;
 	    if(mbnode->mailbox == balsa_app.draftbox)
-		in = BALSA_ICON_DRAFTBOX;
+		in = BALSA_ICON_MBOX_DRAFT;
 	    else if(mbnode->mailbox == balsa_app.inbox)
-		in = BALSA_ICON_INBOX;
+		in = BALSA_ICON_MBOX_IN;
 	    else if(mbnode->mailbox == balsa_app.outbox)
-		in = BALSA_ICON_OUTBOX;
+		in = BALSA_ICON_MBOX_OUT;
 	    else if(mbnode->mailbox == balsa_app.sentbox)
-		in = BALSA_ICON_SENTBOX;
+		in = BALSA_ICON_MBOX_SENT;
 	    else if(mbnode->mailbox == balsa_app.trash)
-		in = BALSA_ICON_TRASH;
+		in = BALSA_ICON_MBOX_TRASH;
 	    else
 		in = (mbnode->mailbox->new_messages > 0)
-		? BALSA_ICON_TRAY_FULL : BALSA_ICON_TRAY_EMPTY;
+		? BALSA_ICON_MBOX_TRAY_FULL : BALSA_ICON_MBOX_TRAY_EMPTY;
 
 	    gtk_ctree_set_node_info(ctree, cnode,
 				    mbnode->mailbox->name, 5,
@@ -734,11 +734,11 @@ mailbox_nodes_to_ctree(GtkCTree * ctree, guint depth, GNode * gnode,
 	/* new directory, but not a mailbox */
 	gtk_ctree_set_node_info(ctree, cnode, g_basename(mbnode->name), 5,
 				balsa_icon_get_pixmap
-				(BALSA_ICON_DIR_CLOSED),
+				(BALSA_ICON_MBOX_DIR_CLOSED),
 				balsa_icon_get_bitmap
-				(BALSA_ICON_DIR_CLOSED),
-				balsa_icon_get_pixmap(BALSA_ICON_DIR_OPEN),
-				balsa_icon_get_bitmap(BALSA_ICON_DIR_OPEN),
+				(BALSA_ICON_MBOX_DIR_CLOSED),
+				balsa_icon_get_pixmap(BALSA_ICON_MBOX_DIR_OPEN),
+				balsa_icon_get_bitmap(BALSA_ICON_MBOX_DIR_OPEN),
 				FALSE, mbnode->expanded);
 	gtk_ctree_node_set_selectable(ctree, cnode, FALSE);
     }
@@ -1032,7 +1032,7 @@ balsa_mblist_update_node_style(GtkCTree * ctree, GtkCTreeNode * node,
 	    tmp_is_leaf = GTK_CTREE_ROW(node)->is_leaf;
 	    tmp_expanded = GTK_CTREE_ROW(node)->expanded;
 	    
-	    icon = BALSA_ICON_TRAY_FULL;
+	    icon = BALSA_ICON_MBOX_TRAY_FULL;
 	    
 	    gtk_ctree_set_node_info(ctree, node, mailbox->name, 5,
 				    balsa_icon_get_pixmap(icon),
@@ -1072,9 +1072,9 @@ balsa_mblist_update_node_style(GtkCTree * ctree, GtkCTreeNode * node,
 		tmp_expanded = GTK_CTREE_ROW(node)->expanded;
 		
 		if (mailbox == balsa_app.inbox)
-		    icon = BALSA_ICON_INBOX;
+		    icon = BALSA_ICON_MBOX_IN;
 		else
-		    icon = BALSA_ICON_TRAY_EMPTY;
+		    icon = BALSA_ICON_MBOX_TRAY_EMPTY;
 		
 		gtk_ctree_set_node_info(ctree, node, mailbox->name, 5,
 					balsa_icon_get_pixmap(icon),
@@ -1368,6 +1368,17 @@ balsa_mblist_focus_mailbox(BalsaMBList * bmbl, LibBalsaMailbox * mailbox)
    remove give mailbox node from the mailbox tree.
    Return TRUE (or equivalent) on success, FALSE on failure.
 */
+void
+mblist_remove_mblist_node(BalsaMBList *mblist, BalsaMailboxNode *mbnode,
+		GtkCTreeNode *node)
+{
+    if (mbnode->mailbox)
+	gtk_signal_disconnect_by_func(GTK_OBJECT(mbnode->mailbox),
+				      balsa_mblist_unread_messages_changed_cb,
+				      (gpointer) GTK_CTREE(mblist));
+    gtk_ctree_remove_node(GTK_CTREE(mblist), node);
+}
+
 gboolean
 mblist_remove_mailbox_node(BalsaMBList *mblist, BalsaMailboxNode* mbnode)
 {
@@ -1377,7 +1388,7 @@ mblist_remove_mailbox_node(BalsaMBList *mblist, BalsaMailboxNode* mbnode)
 
     node = gtk_ctree_find_by_row_data(GTK_CTREE(mblist), NULL, mbnode);
     if(node)
-	gtk_ctree_remove_node(GTK_CTREE(mblist), node);
+	mblist_remove_mblist_node(mblist, mbnode, node);
 
     return node != NULL;
 }
@@ -1520,6 +1531,8 @@ mblist_drag_cb (GtkWidget* widget, GdkDragContext* context,
             switch (context->action) {
             case GDK_ACTION_MOVE:
                 libbalsa_messages_move (messages, mailbox);
+                balsa_index_select_next_threaded
+                    (balsa_find_index_by_mailbox(orig_mailbox));
                 break;
 
             case GDK_ACTION_DEFAULT:
@@ -1528,9 +1541,15 @@ mblist_drag_cb (GtkWidget* widget, GdkDragContext* context,
                 libbalsa_messages_copy (messages, mailbox);
                 break;
             }
-            
+
             libbalsa_mailbox_sync_backend (orig_mailbox);
 	    balsa_mblist_update_mailbox(balsa_app.mblist, mailbox);
+
+	    if (mailbox == balsa_app.trash) {
+		enable_empty_trash(TRASH_FULL);
+	    } else if (orig_mailbox == balsa_app.trash) {
+		enable_empty_trash(TRASH_CHECK);
+	    }
         }
     }
     
@@ -1559,7 +1578,9 @@ mblist_drag_motion_cb(GtkWidget * mblist, GdkDragContext * context,
 
     y -= adjust;
     if (gtk_clist_get_selection_info(GTK_CLIST(mblist), x, y, &row, &col)
-        && row != GPOINTER_TO_INT(GTK_CLIST(mblist)->selection->data)) {
+        && (!GTK_CLIST(mblist)->selection
+            || row !=
+            GPOINTER_TO_INT(GTK_CLIST(mblist)->selection->data))) {
         gtk_signal_handler_block_by_func(GTK_OBJECT(mblist),
                                          GTK_SIGNAL_FUNC
                                          (select_mailbox), NULL);
