@@ -1,4 +1,5 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* vim:set ts=4 sw=4 ai et: */
 /* Balsa E-Mail Client
  * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
@@ -33,6 +34,9 @@
 #include "mime.h"
 #include "misc.h"
 
+#include <libmutt/mutt.h>
+#include <libmutt/mime.h>
+
 #ifdef USE_PIXBUF
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
@@ -108,6 +112,8 @@ static void save_part(BalsaPartInfo * info);
 
 static void select_icon_cb(GnomeIconList * ilist, gint num,
 			   GdkEventButton * event, BalsaMessage * bm);
+static BalsaPartInfo *add_part(BalsaMessage *bm, gint part);
+static void add_multipart(BalsaMessage *bm, LibBalsaMessageBody *parent);
 static void select_part(BalsaMessage * bm, gint part);
 static void part_context_menu_save(GtkWidget * menu_item,
 				   BalsaPartInfo * info);
@@ -132,6 +138,7 @@ static void balsa_gtk_html_size_request(GtkWidget * widget,
 static void balsa_gtk_html_link_clicked(GtkWidget *html, 
 					const gchar *url);
 #endif
+static void balsa_gtk_html_on_url(GtkWidget *html, const gchar *url);
 static void balsa_icon_list_size_request(GtkWidget * widget,
 					 GtkRequisition * requisition,
 					 gpointer data);
@@ -155,12 +162,14 @@ static void part_info_init_html(BalsaMessage * bm, BalsaPartInfo * info,
 				gchar * ptr, size_t len);
 #endif
 static GtkWidget* part_info_mime_button (BalsaPartInfo* info, const gchar* content_type, const gchar* key);
+static void part_context_menu_call_url(GtkWidget * menu_item, BalsaPartInfo * info);
+static void part_context_menu_mail(GtkWidget * menu_item, BalsaPartInfo * info);
 static void part_context_menu_cb(GtkWidget * menu_item, BalsaPartInfo * info);
+static void part_context_menu_vfs_cb(GtkWidget * menu_item, BalsaPartInfo * info);
 static void part_create_menu (BalsaPartInfo* info);
 
 static GtkViewportClass *parent_class = NULL;
 
-#ifdef BALSA_MDN_REPLY
 /* stuff needed for sending Message Disposition Notifications */
 static gboolean rfc2298_address_equal(LibBalsaAddress *a, LibBalsaAddress *b);
 static void handle_mdn_request(LibBalsaMessage *message);
@@ -170,7 +179,6 @@ static GtkWidget* create_mdn_dialog (gchar *sender, gchar *mdn_to_address,
 static void mdn_dialog_delete (GtkWidget *dialog, GdkEvent *event, gpointer user_data);
 static void no_mdn_reply (GtkWidget *widget, gpointer user_data);
 static void send_mdn_reply (GtkWidget *widget, gpointer user_data);
-#endif
 
 static BalsaPartInfo* part_info_new(LibBalsaMessageBody* body,
 				    LibBalsaMessage* msg);
@@ -232,6 +240,13 @@ balsa_message_init(BalsaMessage * bm)
     gtk_container_add(GTK_CONTAINER(bm), bm->table);
     gtk_widget_show(bm->table);
 
+    bm->content = gtk_vbox_new(FALSE, 0);
+
+    gtk_table_attach(GTK_TABLE(bm->table), bm->content, 0, 1, 1,
+		     2, GTK_EXPAND | GTK_FILL,
+		     GTK_EXPAND | GTK_FILL, 0, 1);
+    gtk_widget_show(bm->content);
+
     bm->header_text = gtk_text_new(NULL, NULL);
     gtk_signal_connect(GTK_OBJECT(bm->header_text), "key_press_event",
 		       (GtkSignalFunc) balsa_message_key_press_event,
@@ -246,7 +261,7 @@ balsa_message_init(BalsaMessage * bm)
     bm->part_list = gnome_icon_list_new(100, NULL, FALSE);
 
     gnome_icon_list_set_selection_mode(GNOME_ICON_LIST(bm->part_list),
-				       GTK_SELECTION_SINGLE);
+				       GTK_SELECTION_MULTIPLE);
     gtk_signal_connect(GTK_OBJECT(bm->part_list), "select_icon",
 		       GTK_SIGNAL_FUNC(select_icon_cb), bm);
     gtk_signal_connect(GTK_OBJECT(bm->part_list), "size_request",
@@ -357,6 +372,11 @@ save_part(BalsaPartInfo * info)
 
     save_dialog = 
 	GTK_FILE_SELECTION(gtk_file_selection_new(_("Save MIME Part")));
+    /* start workaround for prematurely realized widget returned
+     * by some GTK+ versions */
+    if(GTK_WIDGET_REALIZED(save_dialog))
+        gtk_widget_unrealize(GTK_WIDGET(save_dialog));
+    /* end workaround for prematurely realized widget */
     gtk_window_set_wmclass(GTK_WINDOW(save_dialog), "save_part", "Balsa");
 
     if (balsa_app.save_dir)
@@ -472,20 +492,12 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
 	return TRUE;
     }
 
-#ifdef BALSA_MDN_REPLY
     /*
      * At this point we check if (a) a message has the unread flag set and
      * (b) a Disposition-Notification-To header line is present
      */
     if (message->flags & LIBBALSA_MESSAGE_FLAG_NEW && message->dispnotify_to)
 	handle_mdn_request (message);
-#endif
-
-    /* mark message as read; no-op if it was read so don't worry.
-       and this is the right place to do the marking.
-       body_ref marks the message as read.
-       libbalsa_message_read(message);
-     */
 
     bm->message = message;
 
@@ -805,11 +817,216 @@ part_info_init_image(BalsaMessage * bm, BalsaPartInfo * info)
 
 }
 
+typedef enum _rfc_extbody_t {
+    RFC2046_EXTBODY_FTP,
+    RFC2046_EXTBODY_ANONFTP,
+    RFC2046_EXTBODY_TFTP,
+    RFC2046_EXTBODY_LOCALFILE,
+    RFC2046_EXTBODY_MAILSERVER,
+    RFC2017_EXTBODY_URL,
+    RFC2046_EXTBODY_UNKNOWN
+} rfc_extbody_t;
+
+typedef struct _rfc_extbody_id {
+    gchar *id_string;
+    rfc_extbody_t action;
+} rfc_extbody_id;
+
+static rfc_extbody_id rfc_extbodys[] = {
+    { "ftp",         RFC2046_EXTBODY_FTP },
+    { "anon-ftp",    RFC2046_EXTBODY_ANONFTP },
+    { "tftp",        RFC2046_EXTBODY_TFTP},
+    { "local-file",  RFC2046_EXTBODY_LOCALFILE},
+    { "mail-server", RFC2046_EXTBODY_MAILSERVER},
+    { "URL",         RFC2017_EXTBODY_URL}, 
+    { NULL,          RFC2046_EXTBODY_UNKNOWN}};
+
+static void
+part_info_init_message_extbody_url(BalsaMessage * bm, BalsaPartInfo * info,
+				   rfc_extbody_t url_type)
+{
+    GtkWidget *vbox;
+    GtkWidget *button;
+    GString *msg = NULL;
+    gchar *url;
+
+    if (url_type == RFC2046_EXTBODY_LOCALFILE) {
+	gchar *local_name;
+
+	local_name = 
+	    libbalsa_message_body_get_parameter(info->body, "name");
+
+	if (!local_name) {
+	    part_info_init_unknown(bm, info);
+	    return;
+	}
+
+	url = g_strdup_printf("file:%s", local_name);
+	msg = g_string_new(_("Content Type: external-body\n"));
+	g_string_sprintfa(msg, _("Access type: local-file\n"));
+	g_string_sprintfa(msg, _("File name: %s"), local_name);
+	g_free(local_name);
+    } else if (url_type == RFC2017_EXTBODY_URL) {
+	gchar *local_name;
+
+	local_name = 
+	    libbalsa_message_body_get_parameter(info->body, "URL");
+
+	if (!local_name) {
+	    part_info_init_unknown(bm, info);
+	    return;
+	}
+
+	url = g_strdup(local_name);
+	msg = g_string_new(_("Content Type: external-body\n"));
+	g_string_sprintfa(msg, _("Access type: URL\n"));
+	g_string_sprintfa(msg, _("URL: %s"), url);
+	g_free(local_name);
+    } else { /* *FTP* */
+	gchar *ftp_dir, *ftp_name, *ftp_site;
+	    
+	ftp_dir = 
+	    libbalsa_message_body_get_parameter(info->body, "directory");
+	ftp_name = 
+	    libbalsa_message_body_get_parameter(info->body, "name");
+	ftp_site = 
+	    libbalsa_message_body_get_parameter(info->body, "site");
+
+	if (!ftp_name || !ftp_site) {
+	    part_info_init_unknown(bm, info);
+	    g_free(ftp_dir);
+	    g_free(ftp_name);
+	    g_free(ftp_site);
+	    return;
+	}
+
+	if (ftp_dir)
+	    url = g_strdup_printf("%s://%s/%s/%s", 
+				  url_type == RFC2046_EXTBODY_TFTP ? "tftp" : "ftp",
+				  ftp_site, ftp_dir, ftp_name);
+	else
+	    url = g_strdup_printf("%s://%s/%s", 
+				  url_type == RFC2046_EXTBODY_TFTP ? "tftp" : "ftp",
+				  ftp_site, ftp_name);
+	msg = g_string_new(_("Content Type: external-body\n"));
+	g_string_sprintfa(msg, _("Access type: %s\n"),
+			  url_type == RFC2046_EXTBODY_TFTP ? "tftp" :
+			  url_type == RFC2046_EXTBODY_FTP ? "ftp" : "anon-ftp");
+	g_string_sprintfa(msg, _("FTP site: %s\n"), ftp_site);
+	if (ftp_dir)
+	    g_string_sprintfa(msg, _("Directory: %s\n"), ftp_dir);
+	g_string_sprintfa(msg, _("File name: %s"), ftp_name);
+	g_free(ftp_dir);
+	g_free(ftp_name);
+	g_free(ftp_site);
+    }
+
+    /* now create the widget... */
+    vbox = gtk_vbox_new(FALSE, 1);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+
+    gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(msg->str), FALSE, FALSE, 1);
+    g_string_free(msg, TRUE);
+
+    button = gtk_button_new_with_label(url);
+    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
+    gtk_object_set_data(GTK_OBJECT(button), "call_url", url);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		       GTK_SIGNAL_FUNC(part_context_menu_call_url),
+		       (gpointer) info);
+
+    gtk_widget_show_all(vbox);
+
+    info->focus_widget = vbox;
+    info->widget = vbox;
+    info->can_display = FALSE;    
+}
+
+static void
+part_info_init_message_extbody_mail(BalsaMessage * bm, BalsaPartInfo * info)
+{
+    GtkWidget *vbox;
+    GtkWidget *button;
+    GString *msg = NULL;
+    gchar *mail_subject, *mail_site;
+	    
+    mail_site =
+	libbalsa_message_body_get_parameter(info->body, "server");
+
+    if (!mail_site) {
+	part_info_init_unknown(bm, info);
+	return;
+    }
+
+    mail_subject =
+	libbalsa_message_body_get_parameter(info->body, "subject");
+
+    msg = g_string_new(_("Content Type: external-body\n"));
+    msg = g_string_append (msg, _("Access type: mail-server\n"));
+    g_string_sprintfa(msg, _("Mail server: %s\n"), mail_site);
+    if (mail_subject)
+	g_string_sprintfa(msg, _("Subject: %s\n"), mail_subject);
+    g_free(mail_subject);
+    g_free(mail_site);
+
+    /* now create the widget... */
+    vbox = gtk_vbox_new(FALSE, 1);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
+
+    gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(msg->str), FALSE, FALSE, 1);
+    g_string_free(msg, TRUE);
+
+    button = gtk_button_new_with_label(_("Send message to obtain this part"));
+    gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		       GTK_SIGNAL_FUNC(part_context_menu_mail),
+		       (gpointer) info);
+
+    gtk_widget_show_all(vbox);
+
+    info->focus_widget = vbox;
+    info->widget = vbox;
+    info->can_display = FALSE;    
+}
+
 static void
 part_info_init_message(BalsaMessage * bm, BalsaPartInfo * info)
 {
-    g_print("TODO: part_info_init_message\n");
-    part_info_init_unknown(bm, info);
+    if (info->body && info->body->mutt_body && info->body->mutt_body->subtype &&
+	!g_strcasecmp("external-body", info->body->mutt_body->subtype)) {
+	gchar *access_type;
+	rfc_extbody_id *extbody_type = rfc_extbodys;
+
+	access_type = 
+	    libbalsa_message_body_get_parameter(info->body, "access-type");
+	while (extbody_type->id_string && 
+	       g_strcasecmp(extbody_type->id_string, access_type))
+	    extbody_type++;
+	switch (extbody_type->action) {
+	case RFC2046_EXTBODY_FTP:
+	case RFC2046_EXTBODY_ANONFTP:
+	case RFC2046_EXTBODY_TFTP:
+	case RFC2046_EXTBODY_LOCALFILE:
+	case RFC2017_EXTBODY_URL:
+	    part_info_init_message_extbody_url(bm, info, extbody_type->action);
+	    break;
+	case RFC2046_EXTBODY_MAILSERVER:
+	    part_info_init_message_extbody_mail(bm, info);
+	    break;
+	case RFC2046_EXTBODY_UNKNOWN:
+	    g_print("TODO: part_info_init_message (external-body, access-type %s)\n",
+		    access_type);
+	    part_info_init_unknown(bm, info);
+	    break;
+	default:
+	    g_error("Undefined external body action %d!", extbody_type->action);
+	    break;
+	}
+	g_free(access_type);
+    } else {
+	g_print("TODO: part_info_init_message\n");
+	part_info_init_unknown(bm, info);
+    }
 }
 
 static void
@@ -1474,6 +1691,7 @@ check_over_url(GtkWidget *widget, GdkEvent *event, gpointer data)
 	    if (!was_over_url) {
 		gdk_window_set_cursor(GTK_TEXT(widget)->text_area, 
 				      url_cursor_over_url);
+		balsa_gtk_html_on_url(NULL, hotarea_data->url->url);
 		was_over_url = TRUE;
 	    }
 	    return FALSE;
@@ -1483,6 +1701,7 @@ check_over_url(GtkWidget *widget, GdkEvent *event, gpointer data)
 
     if (was_over_url) {
 	gdk_window_set_cursor(GTK_TEXT(widget)->text_area, url_cursor_normal);
+	balsa_gtk_html_on_url(NULL, NULL);
 	was_over_url = FALSE;
     }
 
@@ -1603,9 +1822,16 @@ part_info_init_mimetext(BalsaMessage * bm, BalsaPartInfo * info)
 	
 	fnt = find_body_font(info->body);
 
-	if (bm->wrap_text)
-	    libbalsa_wrap_string(ptr, balsa_app.wraplength);
-	
+    if (bm->wrap_text) {
+        if (balsa_app.recognize_rfc2646_format_flowed
+            && libbalsa_flowed_rfc2646(info->body)) {
+            ptr =
+                libbalsa_wrap_rfc2646(ptr, balsa_app.browse_wrap_length,
+                                      FALSE, TRUE);
+        } else
+            libbalsa_wrap_string(ptr, balsa_app.browse_wrap_length);
+    }
+
 	if (!fnt)
 	    fnt = gdk_fontset_load(balsa_app.message_font);
 	
@@ -1717,6 +1943,9 @@ part_info_init_html(BalsaMessage * bm, BalsaPartInfo * info, gchar * ptr,
     gtk_signal_connect(GTK_OBJECT(html), "link_clicked",
 		       GTK_SIGNAL_FUNC(balsa_gtk_html_link_clicked),
 		       bm);
+    gtk_signal_connect(GTK_OBJECT(html), "on_url",
+		       GTK_SIGNAL_FUNC(balsa_gtk_html_on_url),
+		       bm);
 
     gtk_container_add(GTK_CONTAINER(scroll), html);
 
@@ -1768,7 +1997,6 @@ part_info_init(BalsaMessage * bm, BalsaPartInfo * info)
 	fprintf(stderr, "part end: multipart\n");
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_MULTIPART:
-	g_print("Got a TYPEMULTIPART in part2canvas!\n");
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_TEXT:
 	if (balsa_app.debug)
@@ -1799,25 +2027,21 @@ display_part(BalsaMessage * bm, LibBalsaMessageBody * body)
 {
     BalsaPartInfo *info = NULL;
     gchar *pix = NULL;
-    gchar *content_type = NULL;
+    gchar *content_type = libbalsa_message_body_get_content_type(body);
     gchar *icon_title = NULL;
     gint pos;
+    gboolean is_multipart=libbalsa_message_body_is_multipart(body);
 
-
-    if (libbalsa_message_body_is_multipart(body)) {
-	if (balsa_app.debug)
-	    fprintf(stderr, "part: multipart\n");
-	display_multipart(bm, body);
-	if (balsa_app.debug)
-	    fprintf(stderr, "part end: multipart\n");
-	return;			/* we don't want a multipart icon */
-    }
-
+    if(!is_multipart ||
+       g_strcasecmp(content_type, "multipart/mixed")==0 ||
+       g_strcasecmp(content_type, "multipart/alternative")==0) {
     bm->part_count++;
     info = part_info_new(body, bm->message);
-    content_type = libbalsa_message_body_get_content_type(body);
 
-    if (body->filename)
+	if (is_multipart) {
+	    icon_title = g_strdup_printf("%s parts", strchr(content_type, '/')+1);
+	    *icon_title = toupper (*icon_title);
+	} else if (body->filename)
 	icon_title =
 	    g_strdup_printf("%s (%s)", body->filename, content_type);
     else
@@ -1832,9 +2056,17 @@ display_part(BalsaMessage * bm, LibBalsaMessageBody * body)
     gnome_icon_list_set_icon_data_full(GNOME_ICON_LIST(bm->part_list), pos,
 				       info, (GtkDestroyNotify)part_info_free);
 
-    g_free(content_type);
     g_free(icon_title);
     g_free(pix);
+    }
+    if (is_multipart) {
+	if (balsa_app.debug)
+	    fprintf(stderr, "part: multipart\n");
+	display_multipart(bm, body);
+	if (balsa_app.debug)
+	    fprintf(stderr, "part end: multipart\n");
+    }
+    g_free(content_type);
 }
 
 static void
@@ -1869,7 +2101,11 @@ part_create_menu (BalsaPartInfo* info)
         key = list->data;
 
         if (key && g_strcasecmp (key, "icon-filename") 
-	    && g_strncasecmp (key, "fm-", 3)) {
+	    && g_strncasecmp (key, "fm-", 3)
+	    /* Get rid of additional GnomeVFS entries: */
+	    && (!strstr(key, "_") || strstr(key, "."))
+	    && g_strncasecmp(key, "description", 11)) {
+	    
             if ((cmd = gnome_mime_get_value (content_type, key)) != NULL) {
                 if (g_strcasecmp (key, "open") == 0 || 
                     g_strcasecmp (key, "view") == 0 || 
@@ -1956,6 +2192,76 @@ part_context_menu_save(GtkWidget * menu_item, BalsaPartInfo * info)
 }
 
 
+static void
+part_context_menu_call_url(GtkWidget * menu_item, BalsaPartInfo * info)
+{
+    gchar *url = gtk_object_get_data (GTK_OBJECT (menu_item), "call_url");
+    
+    g_return_if_fail(url);
+    gnome_url_show(url);
+}
+
+
+static void
+part_context_menu_mail(GtkWidget * menu_item, BalsaPartInfo * info)
+{
+    LibBalsaMessage *message;
+    LibBalsaMessageBody *body;
+    gchar *data;
+    FILE *part;
+
+    /* create a message */
+    message = libbalsa_message_new();
+    data = libbalsa_address_to_gchar(balsa_app.current_ident->address, 0);
+    message->from = libbalsa_address_new_from_string(data);
+    g_free (data);
+
+    data = libbalsa_message_body_get_parameter(info->body, "subject");
+    if (data)
+	LIBBALSA_MESSAGE_SET_SUBJECT(message, data);
+
+    data = libbalsa_message_body_get_parameter(info->body, "server");
+    message->to_list = libbalsa_address_new_list_from_string(data);
+    g_free (data);
+
+    /* the original body my have some data to be returned as commands... */
+    body = libbalsa_message_body_new(message);
+
+    libbalsa_message_body_save_temporary(info->body, NULL);
+    part = fopen(info->body->temp_filename, "r");
+    if (part) {
+	gchar *p;
+
+	libbalsa_readfile(part, &data);
+	/* ignore everything before the first two newlines */
+	if ((p = strstr (data, "\n\n")))
+	    body->buffer = g_strdup(p + 2);
+	else
+	    body->buffer = g_strdup(data);
+	g_free(data);
+	fclose(part);
+    }
+    if (info->body->charset)
+	body->charset = g_strdup(info->body->charset);
+    else
+	body->charset = g_strdup("US-ASCII");
+    libbalsa_message_append_part(message, body);
+#if ENABLE_ESMTP
+    libbalsa_message_send(message, balsa_app.outbox, NULL,
+			  balsa_app.encoding_style,  
+			  balsa_app.smtp_server,
+			  balsa_app.smtp_authctx,
+			  balsa_app.smtp_tls_mode,
+			  FALSE);
+#else
+    libbalsa_message_send(message, balsa_app.outbox, NULL,
+			  balsa_app.encoding_style,
+			  FALSE);
+#endif
+    gtk_object_destroy(GTK_OBJECT(message));    
+}
+
+
 static void
 part_context_menu_cb(GtkWidget * menu_item, BalsaPartInfo * info)
 {
@@ -2039,6 +2345,156 @@ balsa_message_previous_part(BalsaMessage * bmessage)
     select_part(bmessage, index);
 }
 
+static LibBalsaMessageBody*
+preferred_part(LibBalsaMessageBody *parts)
+{
+    /* TODO: Consult preferences and/or previous selections */
+
+    LibBalsaMessageBody *body;
+    gchar *content_type;
+
+#ifdef HAVE_GTKHTML
+    for(body=parts; body; body=body->next) {
+	content_type = libbalsa_message_body_get_content_type(body);
+
+	if(g_strcasecmp(content_type, "text/html")==0) {
+	    g_free(content_type);
+	    return body;
+	}
+	g_free(content_type);
+    }
+#endif /* HAVE_GTKHTML */
+
+    for(body=parts; body; body=body->next) {
+	content_type = libbalsa_message_body_get_content_type(body);
+
+	if(g_strcasecmp(content_type, "text/plain")==0) {
+	    g_free(content_type);
+	    return body;
+	}
+	g_free(content_type);
+    }
+
+
+    return parts;
+}
+
+
+
+static gint part_icon_no(BalsaMessage *bm, const LibBalsaMessageBody *body)
+{
+    const BalsaPartInfo *info;
+    int part;
+
+    for(part=0; part<bm->part_count; part++) {
+	info = (const BalsaPartInfo *) gnome_icon_list_get_icon_data
+	    (GNOME_ICON_LIST(bm->part_list), part);
+	if(info->body==body)
+	    return part;
+    }
+    return -1;
+}
+
+
+static void add_body(BalsaMessage *bm, 
+		     LibBalsaMessageBody *body)
+{
+    if(body) {
+	gint part=part_icon_no(bm, body);
+	
+	if(part>=0)
+	    add_part(bm, part);
+	else
+	    add_multipart(bm, body);
+    }
+}
+
+
+static void add_multipart(BalsaMessage *bm, LibBalsaMessageBody *parent)
+/* Remarks: *** The tests/assumptions made are NOT verified with the RFCs */
+{
+    if(parent->parts) {
+	gchar *content_type = 
+	    libbalsa_message_body_get_content_type(parent);
+	if(g_strcasecmp(content_type, "multipart/related")==0) {
+	    /* Add the first part */
+	    add_body(bm, parent->parts);
+	} else if(g_strcasecmp(content_type, "multipart/alternative")==0) {
+	    /* Add the most suitable part. */
+	    add_body(bm, preferred_part(parent->parts));
+	} else {
+	    /* Add first (main) part + anything else with 
+	       Content-Disposition: inline */
+	    LibBalsaMessageBody *body=parent->parts;
+	    
+	    if(body) {
+		add_body(bm, body);
+		for(body=body->next; body; body=body->next) {
+		    if(body->mutt_body && 
+		       body->mutt_body->disposition==DISPINLINE)
+			add_body(bm, body);
+		}
+	    }
+	}
+	g_free(content_type);
+    }
+}
+
+
+
+static BalsaPartInfo *add_part(BalsaMessage *bm, gint part)
+{
+    BalsaPartInfo *info=NULL;
+
+    if (part != -1) {
+	LibBalsaMessageBodyType type;
+	
+	info = (BalsaPartInfo *) gnome_icon_list_get_icon_data
+	    (GNOME_ICON_LIST(bm->part_list), part);
+
+	g_assert(info != NULL);
+
+	gnome_icon_list_select_icon(GNOME_ICON_LIST(bm->part_list), part);
+
+	if (info->widget == NULL)
+	    part_info_init(bm, info);
+
+	if (info->widget) {
+	    gtk_container_add(GTK_CONTAINER(bm->content), info->widget);
+	    gtk_widget_show(info->widget);
+	}
+	add_multipart(bm, info->body);
+    }
+    
+    return info;
+}
+
+static void hide_all_parts(BalsaMessage *bm)
+{
+    if(bm->current_part) {
+	gint part;
+
+	for(part=0; part<bm->part_count; part++) {
+	    BalsaPartInfo *current_part=(BalsaPartInfo *) gnome_icon_list_get_icon_data
+		(GNOME_ICON_LIST(bm->part_list), part);
+	    
+	    if(current_part && current_part->widget && 
+	       GTK_WIDGET_VISIBLE(current_part->widget)) {
+		if(GTK_IS_EDITABLE(current_part->widget) && 
+		   GTK_WIDGET_REALIZED(current_part->widget))
+		    gtk_editable_claim_selection(
+						 GTK_EDITABLE(current_part->widget), FALSE, 
+						 GDK_CURRENT_TIME);
+		gtk_widget_hide(current_part->widget);
+		gtk_container_remove(GTK_CONTAINER(bm->content),
+				     current_part->widget);
+	}
+	    gnome_icon_list_unselect_icon(GNOME_ICON_LIST(bm->part_list), part);
+    }
+    }
+}
+
+
 /* 
  * If part == -1 then change to no part
  * must release selection before hiding a text widget.
@@ -2046,52 +2502,13 @@ balsa_message_previous_part(BalsaMessage * bmessage)
 static void
 select_part(BalsaMessage * bm, gint part)
 {
-    BalsaPartInfo *info;
+    hide_all_parts(bm);
 
-    if (bm->current_part && bm->current_part->widget) {
-	if(GTK_IS_EDITABLE(bm->current_part->widget) && 
-	   GTK_WIDGET_REALIZED(bm->current_part->widget))
-	    gtk_editable_claim_selection(
-		GTK_EDITABLE(bm->current_part->widget), FALSE, 
-		GDK_CURRENT_TIME);
-	gtk_widget_hide(bm->current_part->widget);
-	gtk_container_remove(GTK_CONTAINER(bm->table),
-			     bm->current_part->widget);
-    }
+    bm->current_part = add_part(bm, part);
 
-    if (part != -1) {
-	info = (BalsaPartInfo *) gnome_icon_list_get_icon_data
-	    (GNOME_ICON_LIST(bm->part_list), part);
-
-	g_assert(info != NULL);
-
-	if (info->widget == NULL)
-	    part_info_init(bm, info);
-
-	if (info->widget) {
-	    gtk_widget_show(info->widget);
-	    gtk_table_attach(GTK_TABLE(bm->table), info->widget, 0, 1, 1,
-			     2, GTK_EXPAND | GTK_FILL,
-			     GTK_EXPAND | GTK_FILL, 0, 1);
-	} else {
-	    /* HACK! This is a spacer, so that the attachment icons
-	     * will stay at the bottom of the window.
-	     */
-	    GtkWidget *box;
-
-	    box = gtk_hbox_new(FALSE, FALSE);
-	    gtk_widget_show(box);
-	    gtk_table_attach(GTK_TABLE(bm->table), box, 0, 1, 1, 2,
-			     GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
-			     0, 0);
-	}
-	bm->current_part = info;
+    if(bm->current_part)
 	gtk_signal_emit(GTK_OBJECT(bm),
 			balsa_message_signals[SELECT_PART]);
-    } else {
-	bm->current_part = NULL;
-    }
-
 
     scroll_set(GTK_VIEWPORT(bm)->hadjustment, 0);
     scroll_set(GTK_VIEWPORT(bm)->vadjustment, 0);
@@ -2206,6 +2623,14 @@ balsa_gtk_text_size_request(GtkWidget * widget,
 }
 
 #ifdef HAVE_GTKHTML
+/* balsa_gtk_html_size_request:
+   report the requested size of the HTML widget.
+
+   FIXME: this is not 100% right. The code includes an empirical
+   (hehe) term -1 (marked with EMP) which is NOT the right way to
+   go. The right solution requires some study of size_request signal
+   handling code.  
+*/
 static void
 balsa_gtk_html_size_request(GtkWidget * widget,
 			    GtkRequisition * requisition, gpointer data)
@@ -2214,11 +2639,11 @@ balsa_gtk_html_size_request(GtkWidget * widget,
     g_return_if_fail(GTK_IS_HTML(widget));
     g_return_if_fail(requisition != NULL);
 
-    requisition->width = (widget->style->klass->xthickness + 1) * 2;
-    requisition->height = (widget->style->klass->ythickness + 1) * 2;
+    requisition->width  = -(widget->style->klass->xthickness + 1) * 2;
+    requisition->height = -(widget->style->klass->ythickness + 1) * 2;
 
-    requisition->width += GTK_LAYOUT(widget)->hadjustment->upper;
-    requisition->height += GTK_LAYOUT(widget)->vadjustment->upper;
+    requisition->width  += GTK_LAYOUT(widget)->hadjustment->upper -1 /*EMP*/;
+    requisition->height += GTK_LAYOUT(widget)->vadjustment->upper -1 /*EMP*/;
 
 }
 
@@ -2228,6 +2653,27 @@ balsa_gtk_html_link_clicked(GtkWidget *html, const gchar *url)
     gnome_url_show(url);
 }
 #endif
+static void
+balsa_gtk_html_on_url(GtkWidget *html, const gchar *url)
+{
+    static url_pushed = FALSE;
+
+    if( url ) {
+	if (url_pushed) {
+	    gnome_appbar_set_status(balsa_app.appbar, url);
+	} else {
+	    gnome_appbar_push(balsa_app.appbar, url);
+	    url_pushed = TRUE;
+	}
+    } else {
+	if (url_pushed) {
+	    gnome_appbar_pop(balsa_app.appbar);
+	    url_pushed = FALSE;
+	} else {
+	    gnome_appbar_set_status(balsa_app.appbar, "");
+	}
+    }
+}
 
 static void
 balsa_icon_list_size_request(GtkWidget * widget,
@@ -2303,7 +2749,6 @@ balsa_message_select_all(BalsaMessage * bmessage)
 			       -1);
 }
 
-#ifdef BALSA_MDN_REPLY
 /* rfc2298_address_equal
    compares two addresses according to rfc2298: local-part@domain is equal,
    if the local-parts are case sensitive equal, but the domain case-insensitive
@@ -2415,10 +2860,12 @@ handle_mdn_request(LibBalsaMessage *message)
 			      balsa_app.encoding_style,  
 			      balsa_app.smtp_server,
 			      balsa_app.smtp_authctx,
-			      balsa_app.smtp_tls_mode);
+			      balsa_app.smtp_tls_mode,
+			      balsa_app.send_rfc2646_format_flowed);
 #else
 	libbalsa_message_send(mdn, balsa_app.outbox, NULL,
-			      balsa_app.encoding_style);
+			      balsa_app.encoding_style,
+			      balsa_app.send_rfc2646_format_flowed);
 #endif
 	gtk_object_destroy(GTK_OBJECT(mdn));
     }
@@ -2575,13 +3022,14 @@ static void send_mdn_reply (GtkWidget *widget, gpointer user_data)
 			  balsa_app.encoding_style,  
 			  balsa_app.smtp_server,
 			  balsa_app.smtp_authctx,
-			  balsa_app.smtp_tls_mode);
+			  balsa_app.smtp_tls_mode,
+			  balsa_app.send_rfc2646_format_flowed);
 #else
     libbalsa_message_send(send_msg, balsa_app.outbox, NULL,
-			  balsa_app.encoding_style);  
+			  balsa_app.encoding_style,
+			  balsa_app.send_rfc2646_format_flowed);
 #endif
     gtk_object_destroy(GTK_OBJECT(send_msg));
     gtk_widget_hide (dialog);
     gtk_object_destroy(GTK_OBJECT(dialog));
 }
-#endif
