@@ -301,6 +301,9 @@ balsa_mailbox_node_new_from_config(const gchar* prefix)
     printf("Server loaded, host: %s, port %d\n", folder->server->host,
 	   folder->server->port);
 #endif
+    gtk_signal_connect(GTK_OBJECT(folder->server),
+		       "get-password", GTK_SIGNAL_FUNC(ask_password),
+		       NULL);
     gtk_signal_connect(GTK_OBJECT(folder), "show-prop-dialog", 
 		       folder_conf_imap_node, NULL);
     gtk_signal_connect(GTK_OBJECT(folder), "append-subtree", 
@@ -816,11 +819,13 @@ add_imap_entry(GNode*root, const char* fn, char delim)
     gchar* url;
 
     node = balsa_app_find_by_dir(root, fn);
-    if (node != root)
+    if (node && node != root)
 	return node;
 
     parent_name = get_parent_folder_name(fn, delim);
     parent = balsa_app_find_by_dir(root, parent_name);
+    if (parent == NULL)
+        parent = root;
     g_free(parent_name);
 
     g_return_val_if_fail(parent, NULL);
