@@ -1,4 +1,5 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
+/* vim:set ts=4 sw=4 ai et: */
 /* Balsa E-Mail Client
  * Copyright (C) 1997-2001 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
@@ -50,6 +51,8 @@ static void config_address_books_load(void);
 static void config_address_books_save(void);
 static void config_identities_load(void);
 
+static void check_for_old_sigs(GList * id_list_tmp);
+
 #define folder_section_path(mn) \
     BALSA_MAILBOX_NODE(mn)->config_prefix ? \
     g_strdup(BALSA_MAILBOX_NODE(mn)->config_prefix) : \
@@ -84,6 +87,14 @@ d_get_gint(const gchar * key, gint def_val)
     return def ? def_val : res;
 }
 
+static void
+free_toolbar(int i)
+{
+    int j;
+    for(j=0; balsa_app.toolbars[i][j]; j++)
+	g_free(balsa_app.toolbars[i][j]);
+    g_free(balsa_app.toolbars[i]);
+}
 static void
 free_toolbars(void)
 {
@@ -92,13 +103,10 @@ free_toolbars(void)
     if(!balsa_app.toolbars == 0)
 	return;
     
-    for(i=0; i<balsa_app.toolbar_count; i++) {
-	for(j=0; balsa_app.toolbars[i][j]; j++)
-	    g_free(balsa_app.toolbars[i][j]);
-	free((char *)balsa_app.toolbars[i]);
-    }
-    free((void *)balsa_app.toolbars);
-    free((void *)balsa_app.toolbar_ids);
+    for(i=0; i<balsa_app.toolbar_count; i++) 
+	free_toolbar(i);
+    g_free((void *)balsa_app.toolbars);
+    g_free((void *)balsa_app.toolbar_ids);
 }
 
 /* load_toolbars:
@@ -117,33 +125,43 @@ load_toolbars(void)
     free_toolbars();
     
     balsa_app.toolbar_ids=
-	(int *)g_malloc(sizeof(int)*MAXTOOLBARS);
+	(BalsaToolbarType *)g_malloc(sizeof(BalsaToolbarType)*MAXTOOLBARS);
     balsa_app.toolbars=	(char ***)g_malloc(sizeof(char *)*MAXTOOLBARS);
     
     balsa_app.toolbar_count=d_get_gint("ToolbarCount", 0);
-    if(balsa_app.toolbar_count)	{
-	for(i=0; i<balsa_app.toolbar_count; i++) {
-	    balsa_app.toolbars[i]=
-		(char **)g_malloc(sizeof(char *)*MAXTOOLBARITEMS);
-			
-	    sprintf(tmpkey, "Toolbar%dID", i);
-	    balsa_app.toolbar_ids[i] = d_get_gint(tmpkey, -1);
-
-	    if(balsa_app.toolbar_ids[i] == -1) {
-		balsa_app.toolbars[i][0]=NULL;
-		continue;
-	    }
-
-	    sprintf(tmpkey, "Toolbar%dItemCount", i);
-	    items=d_get_gint(tmpkey, 0);
-	    
-	    for(j=0; j<items; j++) {
-		sprintf(tmpkey, "Toolbar%dItem%d", i, j);
-		balsa_app.toolbars[i][j]=  gnome_config_get_string(tmpkey);
-	    }
-	    balsa_app.toolbars[i][j]=NULL;
+    for(i=0; i<balsa_app.toolbar_count; i++) {
+	balsa_app.toolbars[i]=
+	    (char **)g_malloc(sizeof(char *)*MAXTOOLBARITEMS);
+	
+	sprintf(tmpkey, "Toolbar%dID", i);
+	balsa_app.toolbar_ids[i] = d_get_gint(tmpkey, TOOLBAR_INVALID);
+	
+	if(balsa_app.toolbar_ids[i] == TOOLBAR_INVALID) {
+	    balsa_app.toolbars[i][0]=NULL;
+	    continue;
 	}
 	
+	sprintf(tmpkey, "Toolbar%dItemCount", i);
+	items=d_get_gint(tmpkey, 0);
+	
+	for(j=0; j<items; j++) {
+	    sprintf(tmpkey, "Toolbar%dItem%d", i, j);
+	    balsa_app.toolbars[i][j]=  gnome_config_get_string(tmpkey);
+	}
+	balsa_app.toolbars[i][j]=NULL;
+	/* validate */
+	for(j=0; balsa_app.toolbars[i][j]; j++) {
+	    if(get_toolbar_button_index(balsa_app.toolbars[i][j])<0) {
+		/* validation failed: roll the toolbar back. */
+		free_toolbar(i); 
+		balsa_app.toolbars[i] = NULL;
+		balsa_app.toolbar_ids[i] = TOOLBAR_INVALID;
+		i--;
+		balsa_app.toolbar_count--;
+		g_warning("I dropped a toolbar. Are you up/downgrading?");
+		break;
+	    }
+	}
     }
     gnome_config_pop_prefix();
 }
@@ -300,6 +318,7 @@ gint config_mailbox_update(LibBalsaMailbox * mailbox)
     res = gnome_config_has_section(key);
     gnome_config_push_prefix(key);
     libbalsa_mailbox_save_config(mailbox, key);
+    g_free(key);
     gnome_config_pop_prefix();
     gnome_config_sync();
     return res;
@@ -315,6 +334,7 @@ gint config_folder_update(BalsaMailboxNode * mbnode)
     res = gnome_config_has_section(key);
     gnome_config_push_prefix(key);
     balsa_mailbox_node_save_config(mbnode, key);
+    g_free(key);
     gnome_config_pop_prefix();
     gnome_config_sync();
     return res;
@@ -422,7 +442,10 @@ config_global_load(void)
 #endif
     config_address_books_load();
     config_identities_load();
-    
+
+    /* find and convert old-style signature entries */
+    check_for_old_sigs(balsa_app.identities);
+
     /* Section for the balsa_information() settings... */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "InformationMessages/");
 
@@ -505,6 +528,8 @@ config_global_load(void)
     g_free(balsa_app.quote_regex);
     balsa_app.quote_regex =
 	gnome_config_get_string("QuoteRegex=" DEFAULT_QUOTE_REGEX);
+    balsa_app.recognize_rfc2646_format_flowed =
+	gnome_config_get_bool("RecognizeRFC2646FormatFlowed=true");
 
     {
 	int i;
@@ -535,6 +560,9 @@ config_global_load(void)
 
     /* ... wrap words */
     balsa_app.browse_wrap = gnome_config_get_bool("WordWrap=true");
+    balsa_app.browse_wrap_length = gnome_config_get_int("WordWrapLength=79");
+    if (balsa_app.browse_wrap_length < 40)
+	balsa_app.browse_wrap_length = 40;
 
     gnome_config_pop_prefix();
 
@@ -620,14 +648,11 @@ config_global_load(void)
     balsa_app.quiet_background_check=d_get_gint("QuietBackgroundCheck", 0);
     gnome_config_pop_prefix();
 
-#ifdef BALSA_MDN_REPLY
     /* how to react if a message with MDN request is displayed */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "MDNReply/");
     balsa_app.mdn_reply_clean = gnome_config_get_int("Clean=1");
     balsa_app.mdn_reply_notclean = gnome_config_get_int("Suspicious=0");
-
     gnome_config_pop_prefix();
-#endif
 
     /* Sending options ... */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "Sending/");
@@ -661,13 +686,20 @@ config_global_load(void)
     /* ... outgoing mail */
     balsa_app.encoding_style = gnome_config_get_int("EncodingStyle=2");
     balsa_app.wordwrap = gnome_config_get_bool("WordWrap=true");
-    balsa_app.wraplength = gnome_config_get_int("WrapLength=75");
+    balsa_app.wraplength = gnome_config_get_int("WrapLength=72");
     if (balsa_app.wraplength < 40)
 	balsa_app.wraplength = 40;
+    balsa_app.send_rfc2646_format_flowed =
+	gnome_config_get_bool("SendRFC2646FormatFlowed=true");
+    balsa_app.autoquote = 
+	gnome_config_get_bool("AutoQuote=true");
     balsa_app.reply_strip_html = 
 	gnome_config_get_bool("StripHtmlInReply=true");
+    balsa_app.forward_attached = 
+	gnome_config_get_bool("ForwardAttached=true");
 
 	balsa_app.always_queue_sent_mail = d_get_gint("AlwaysQueueSentMail", 0);
+	balsa_app.copy_to_sentbox = d_get_gint("CopyToSentbox", 1);
 
     gnome_config_pop_prefix();
 
@@ -698,6 +730,8 @@ config_global_load(void)
     balsa_app.mailbox_nodes = g_node_new(balsa_mailbox_node_new_from_dir(
         balsa_app.local_mail_directory));
 
+    balsa_app.open_inbox_upon_startup =
+	gnome_config_get_bool("OpenInboxOnStartup=false");
     /* debugging enabled */
     balsa_app.debug = gnome_config_get_bool("Debug=false");
 
@@ -799,9 +833,12 @@ gint config_save(void)
     gnome_config_set_string("SelectedHeaders", balsa_app.selected_headers);
     gnome_config_set_int("ThreadingType", balsa_app.threading_type);
     gnome_config_set_string("QuoteRegex", balsa_app.quote_regex);
+    gnome_config_set_bool("RecognizeRFC2646FormatFlowed", 
+			  balsa_app.recognize_rfc2646_format_flowed);
     gnome_config_set_string("MessageFont", balsa_app.message_font);
     gnome_config_set_string("SubjectFont", balsa_app.subject_font);
     gnome_config_set_bool("WordWrap", balsa_app.browse_wrap);
+    gnome_config_set_int("WordWrapLength", balsa_app.browse_wrap_length);
 
     for(i=0;i<MAX_QUOTED_COLOR;i++) {
 	gchar *text = g_strdup_printf("QuotedColor%d", i);
@@ -872,14 +909,11 @@ gint config_save(void)
 
     gnome_config_pop_prefix();
 
-#ifdef BALSA_MDN_REPLY
     /* how to react if a message with MDN request is displayed */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "MDNReply/");
     gnome_config_set_int("Clean", balsa_app.mdn_reply_clean);
     gnome_config_set_int("Suspicious", balsa_app.mdn_reply_notclean);
-
     gnome_config_pop_prefix();
-#endif
 
     /* Sending options ... */
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "Sending/");
@@ -895,9 +929,14 @@ gint config_save(void)
    gnome_config_set_int("EncodingStyle", balsa_app.encoding_style);
     gnome_config_set_bool("WordWrap", balsa_app.wordwrap);
     gnome_config_set_int("WrapLength", balsa_app.wraplength);
+    gnome_config_set_bool("SendRFC2646FormatFlowed",
+			   balsa_app.send_rfc2646_format_flowed);
+    gnome_config_set_bool("AutoQuote", balsa_app.autoquote);
     gnome_config_set_bool("StripHtmlInReply", balsa_app.reply_strip_html);
+    gnome_config_set_bool("ForwardAttached", balsa_app.forward_attached);
 
 	gnome_config_set_int("AlwaysQueueSentMail", balsa_app.always_queue_sent_mail);
+	gnome_config_set_int("CopyToSentbox", balsa_app.copy_to_sentbox);
     gnome_config_pop_prefix();
 
     /* Compose window ... */
@@ -914,6 +953,8 @@ gint config_save(void)
 
     gnome_config_set_string("MailDir", balsa_app.local_mail_directory);
 
+    gnome_config_set_bool("OpenInboxOnStartup", 
+                          balsa_app.open_inbox_upon_startup);
     gnome_config_set_bool("Debug", balsa_app.debug);
 
     gnome_config_set_bool("AutoCloseMailbox", balsa_app.close_mailbox_auto);
@@ -941,28 +982,21 @@ gint config_save(void)
     gnome_config_pop_prefix();
 
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "Toolbars/");
+    gnome_config_set_int("WrapButtonText", balsa_app.toolbar_wrap_button_text);
 
-	gnome_config_set_int("WrapButtonText", balsa_app.toolbar_wrap_button_text);
-
-	gnome_config_set_int("ToolbarCount", balsa_app.toolbar_count);
-
-	if(balsa_app.toolbar_count > 0)
-	{
-		for(i=0;i<balsa_app.toolbar_count;i++)
-		{
-			sprintf(tmpkey, "Toolbar%dID", i);
-			gnome_config_set_int(tmpkey, balsa_app.toolbar_ids[i]);
-
-			for(j=0;balsa_app.toolbars[i][j];j++)
-			{
-				sprintf(tmpkey, "Toolbar%dItem%d", i, j);
-				gnome_config_set_string(tmpkey, balsa_app.toolbars[i][j]);
-			}
-			sprintf(tmpkey, "Toolbar%dItemCount", i);
-			gnome_config_set_int(tmpkey, j);
-		}
-	}
-
+    gnome_config_set_int("ToolbarCount", balsa_app.toolbar_count);
+    
+    for(i=0; i<balsa_app.toolbar_count; i++) {
+        sprintf(tmpkey, "Toolbar%dID", i);
+        gnome_config_set_int(tmpkey, balsa_app.toolbar_ids[i]);
+        
+        for(j=0;balsa_app.toolbars[i][j];j++) {
+            sprintf(tmpkey, "Toolbar%dItem%d", i, j);
+            gnome_config_set_string(tmpkey, balsa_app.toolbars[i][j]);
+        }
+        sprintf(tmpkey, "Toolbar%dItemCount", i);
+        gnome_config_set_int(tmpkey, j);
+    }
     gnome_config_pop_prefix();
     
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "Paths/");
@@ -1080,7 +1114,7 @@ config_identities_load()
 	if (strncmp(key, IDENTITY_SECTION_PREFIX, pref_len) == 0) {
 	    tmp = g_strconcat(BALSA_CONFIG_PREFIX, key, "/", NULL);
 	    ident = libbalsa_identity_new_config(tmp, key+pref_len);
-	    balsa_app.identities = g_list_append(balsa_app.identities, ident);
+	    balsa_app.identities = g_list_prepend(balsa_app.identities, ident);
 	    g_free(tmp);
 	    if(g_strcasecmp(default_ident, ident->identity_name) == 0)
 		balsa_app.current_ident = ident;
@@ -1120,14 +1154,8 @@ config_identities_save(void)
     conf_vec = g_malloc(sizeof(gchar*) * g_list_length(balsa_app.identities));
 
     g_assert(conf_vec != NULL);
-
-    for (list = balsa_app.identities; list; list = g_list_next(list)) {
-        ident = LIBBALSA_IDENTITY(list->data);
-        conf_vec[i++] = ident->identity_name;
-    }
     
     gnome_config_push_prefix(BALSA_CONFIG_PREFIX "identity/");
-    gnome_config_set_vector("Identities", i, (const char**) conf_vec);
     gnome_config_set_string("CurrentIdentity", 
                             balsa_app.current_ident->identity_name);
     gnome_config_pop_prefix();
@@ -1225,3 +1253,39 @@ save_mru(GList *mru)
     }
     gnome_config_set_int("MRUCount", i);
 }
+
+/* check_for_old_sigs:
+   function for old style signature conversion (executable sigs prefixed
+   with '|') to new style (filename and a checkbox).
+   this function just strips the leading '|'.
+*/
+static void 
+check_for_old_sigs(GList * id_list_tmp)
+{
+    /* strip pipes and spaces,set executable flag if warranted */
+    /* FIXME remove after a few stable releases.*/
+    
+    LibBalsaIdentity* id_tmp = NULL;
+    
+    for (id_list_tmp = balsa_app.identities; id_list_tmp; 
+         id_list_tmp = g_list_next(id_list_tmp)) {
+       
+        id_tmp = LIBBALSA_IDENTITY(id_list_tmp->data);
+        if(!id_tmp->signature_path) continue;
+
+        id_tmp->signature_path = g_strstrip(id_tmp->signature_path);
+        if(*id_tmp->signature_path == '|'){
+            printf("Found old style signature for identity: %s\n"\
+                   "Converting: %s --> ", id_tmp->identity_name, 
+                   id_tmp->signature_path);
+            id_tmp->signature_path = g_strchug(id_tmp->signature_path+1);
+            printf("%s \n", id_tmp->signature_path);
+            
+            /* set new-style executable var*/
+            id_tmp->sig_executable=TRUE;
+            printf("Setting converted signature as executable.\n");
+        }
+    }
+} 
+                                                               
+
