@@ -64,7 +64,7 @@ struct toolbar_bdata {
     GtkWidget *widget;
     void (*callback)(GtkWidget *, gpointer);
     gpointer data;
-    char *id;
+    const char *id;
     int disabled;
     int position;
 } toolbar_data[MAXTOOLBARS][MAXTOOLBARITEMS] = { { {0} } };
@@ -79,55 +79,126 @@ static int toolbar_map_entries=0;
 
 static char *toolbar0_legal[]={
     "",
-    GNOME_STOCK_PIXMAP_MAIL_RCV,
-    GNOME_STOCK_PIXMAP_TRASH,
-    GNOME_STOCK_PIXMAP_MAIL_NEW,
-    GNOME_STOCK_PIXMAP_MAIL,
-    GNOME_STOCK_PIXMAP_MAIL_RPL,
-    BALSA_PIXMAP_MAIL_RPL_ALL,
-    GNOME_STOCK_PIXMAP_MAIL_FWD,
-    GNOME_STOCK_PIXMAP_BACK,
-    GNOME_STOCK_PIXMAP_FORWARD,
+    BALSA_PIXMAP_CLOSE_MBOX,
+    BALSA_PIXMAP_CONTINUE,
+    BALSA_PIXMAP_FORWARD,
+    BALSA_PIXMAP_MARKED_ALL,
+    BALSA_PIXMAP_MARKED_NEW,
+    BALSA_PIXMAP_NEW,
+    BALSA_PIXMAP_NEXT,
+    BALSA_PIXMAP_NEXT_FLAGGED,
     BALSA_PIXMAP_NEXT_UNREAD,
-    GNOME_STOCK_PIXMAP_PRINT,
-    BALSA_PIXMAP_FLAG_UNREAD,
-    BALSA_PIXMAP_MARK_ALL_MSGS,
-    BALSA_PIXMAP_SHOW_ALL_HEADERS,
+    BALSA_PIXMAP_PREVIOUS,
+    BALSA_PIXMAP_PRINT,
+    BALSA_PIXMAP_RECEIVE,
+    BALSA_PIXMAP_REPLY,
+    BALSA_PIXMAP_REPLY_ALL,
+    BALSA_PIXMAP_REPLY_GROUP,
+    BALSA_PIXMAP_SEND_RECEIVE,
+    BALSA_PIXMAP_SHOW_HEADERS,
+    BALSA_PIXMAP_SHOW_PREVIEW,
+    BALSA_PIXMAP_TRASH,
+    BALSA_PIXMAP_TRASH_EMPTY,
     NULL
 };
 
 static char *toolbar1_legal[]={
     "",
-    GNOME_STOCK_PIXMAP_MAIL_SND,
-    GNOME_STOCK_PIXMAP_ATTACH,
-    GNOME_STOCK_PIXMAP_SAVE,
+    BALSA_PIXMAP_ATTACHMENT,
     BALSA_PIXMAP_IDENTITY,
-    GNOME_STOCK_PIXMAP_SPELLCHECK,
-    GNOME_STOCK_PIXMAP_PRINT,
+    BALSA_PIXMAP_POSTPONE,
+    BALSA_PIXMAP_PRINT,
+    BALSA_PIXMAP_SAVE,
+    BALSA_PIXMAP_SEND,
     GNOME_STOCK_PIXMAP_CLOSE,
+    GNOME_STOCK_PIXMAP_SPELLCHECK,
     NULL
 };
 
 static char *toolbar2_legal[]={
     "",
-    GNOME_STOCK_PIXMAP_MAIL_RPL,
-    BALSA_PIXMAP_MAIL_RPL_ALL,
-    GNOME_STOCK_PIXMAP_MAIL_FWD,
-    GNOME_STOCK_PIXMAP_BACK,
-    GNOME_STOCK_PIXMAP_FORWARD,
+    BALSA_PIXMAP_FORWARD,
+    BALSA_PIXMAP_NEXT,
+    BALSA_PIXMAP_NEXT_FLAGGED,
     BALSA_PIXMAP_NEXT_UNREAD,
-    GNOME_STOCK_PIXMAP_TRASH,
-    GNOME_STOCK_PIXMAP_PRINT,
-    GNOME_STOCK_PIXMAP_SAVE,
+    BALSA_PIXMAP_PREVIOUS,
+    BALSA_PIXMAP_PRINT,
+    BALSA_PIXMAP_REPLY,
+    BALSA_PIXMAP_REPLY_ALL,
+    BALSA_PIXMAP_REPLY_GROUP,
+    BALSA_PIXMAP_SAVE,
+    BALSA_PIXMAP_SHOW_HEADERS,
+    BALSA_PIXMAP_TRASH,
+    BALSA_PIXMAP_TRASH_EMPTY,
     GNOME_STOCK_PIXMAP_CLOSE,
-    BALSA_PIXMAP_SHOW_ALL_HEADERS,
     NULL
 };
 
 static char **toolbar_legal[]={toolbar0_legal, toolbar1_legal, toolbar2_legal};
 
-static void populate_stock_toolbar(int bar, int id);
-static int get_toolbar_button_slot(BalsaToolbarType toolbar, char *id);
+button_data toolbar_buttons[]={
+    {"", N_("Separator"), "", 0},
+    {BALSA_PIXMAP_RECEIVE, N_("Check"),
+     N_("Check for new email"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_NEW, N_("Compose"),
+     N_("Compose message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_CONTINUE, N_("Continue"),
+     N_("Continue message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_REPLY, N_("Reply"),
+     N_("Reply to the current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_REPLY_ALL, N_("Reply\nto all"),
+     N_("Reply to all recipients"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_REPLY_GROUP, N_("Reply\nto group"),
+     N_("Reply to mailing list"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_FORWARD, N_("Forward"),
+     N_("Forward the current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_PREVIOUS, N_("Previous"),
+     N_("Open previous"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_NEXT, N_("Next"),
+     N_("Open next"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_NEXT_UNREAD, N_("Next\nunread"),
+     N_("Open next unread message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_NEXT_FLAGGED, N_("Next\nflagged"),
+     N_("Open next flagged message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_TRASH, N_("Trash /\nDelete"),
+     N_("Move the current message to trash"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_POSTPONE, N_("Postpone"),
+     N_("Postpone current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_PRINT, N_("Print"),
+     N_("Print current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_SEND, N_("Send"),
+     N_("Send this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_SEND_RECEIVE, N_("Send /\nReceive"),
+     N_("Send and Receive messages"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_ATTACHMENT, N_("Attach"),
+     N_("Add attachments to this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_SAVE, N_("Save"),
+     N_("Save the current item"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_IDENTITY, N_("Select Identity"),
+     N_("Set identity to use for this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {GNOME_STOCK_PIXMAP_SPELLCHECK, N_("Spelling"),
+     N_("Run a spell check"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {GNOME_STOCK_PIXMAP_CLOSE, N_("Cancel"), 
+     N_("Cancel this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_MARKED_NEW, N_("Toggle\nnew"),
+     N_("Toggle new message flag"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_MARKED_ALL, N_("Mark all"),
+     N_("Mark all messages in current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_SHOW_HEADERS, N_("All\nheaders"),
+     N_("Show all headers"), TOOLBAR_BUTTON_TYPE_TOGGLE},
+    {BALSA_PIXMAP_TRASH_EMPTY, N_("Empty Trash"),
+     N_("Delete messages from the trash mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_CLOSE_MBOX, N_("Close"),
+     N_("Close current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
+    {BALSA_PIXMAP_SHOW_PREVIEW, N_("Preview\npane"),
+     N_("Show preview pane"), TOOLBAR_BUTTON_TYPE_TOGGLE}
+};
+
+const int toolbar_button_count = sizeof(toolbar_buttons)
+     /sizeof(button_data);
+
+static void populate_stock_toolbar(int bar, BalsaToolbarType id);
+static int get_toolbar_button_slot(BalsaToolbarType toolbar, const char *id);
 static GtkToolbar *get_bar_instance(GtkWidget *window, 
 				    BalsaToolbarType toolbar);
 static int get_position_value(BalsaToolbarType toolbar, char *id);
@@ -203,21 +274,19 @@ get_tool_widget(GtkWidget *window, BalsaToolbarType toolbar, char *id)
 	return NULL;
     
     lp=children=gtk_container_children(GTK_CONTAINER(bar));
-	if(!children)
+    if(!children)
 	return(NULL);
 
-    while(position-- && lp) {
-	lp=g_list_next(lp);
-	}
+    lp = g_list_nth(lp, position);
 
     if(!lp) {
-		g_list_free(children);
+	g_list_free(children);
 	return NULL;
-	}
+    }
 	
     child=(GtkWidget *)(lp->data);
 
-	if(children)
+    if(children)
 	g_list_free(children);
 	
     if(!child)
@@ -253,53 +322,76 @@ get_bar_instance(GtkWidget *window, BalsaToolbarType toolbar)
 }
 
 static const gchar* main_toolbar[] = {
-    GNOME_STOCK_PIXMAP_MAIL_RCV, "", 
-    GNOME_STOCK_PIXMAP_TRASH   , "",
-    GNOME_STOCK_PIXMAP_MAIL_NEW, GNOME_STOCK_PIXMAP_MAIL,
-    GNOME_STOCK_PIXMAP_MAIL_RPL, BALSA_PIXMAP_MAIL_RPL_ALL,
-    GNOME_STOCK_PIXMAP_MAIL_FWD, "", GNOME_STOCK_PIXMAP_BACK,
-    GNOME_STOCK_PIXMAP_FORWARD,  BALSA_PIXMAP_NEXT_UNREAD,    
-    "",                          GNOME_STOCK_PIXMAP_PRINT,
+    BALSA_PIXMAP_RECEIVE,
+    "",
+    BALSA_PIXMAP_TRASH,
+    "",
+    BALSA_PIXMAP_NEW,
+    BALSA_PIXMAP_CONTINUE,
+    BALSA_PIXMAP_REPLY,
+    BALSA_PIXMAP_REPLY_ALL,
+    BALSA_PIXMAP_FORWARD,
+    "",
+    BALSA_PIXMAP_PREVIOUS,
+    BALSA_PIXMAP_NEXT,
+    BALSA_PIXMAP_NEXT_UNREAD,
+    "",
+    BALSA_PIXMAP_PRINT,
     NULL
 };
 
 static const gchar* compose_toolbar[] = {
-    GNOME_STOCK_PIXMAP_MAIL_SND,   "", 
-    GNOME_STOCK_PIXMAP_ATTACH,     "",
-    GNOME_STOCK_PIXMAP_SAVE,       "",
-    BALSA_PIXMAP_IDENTITY,         "",
-    GNOME_STOCK_PIXMAP_SPELLCHECK, "",
-    GNOME_STOCK_PIXMAP_PRINT,      "",
-    GNOME_STOCK_PIXMAP_CLOSE, NULL
+    BALSA_PIXMAP_SEND,
+    "",
+    BALSA_PIXMAP_ATTACHMENT,
+    "",
+    BALSA_PIXMAP_SAVE,
+    "",
+    BALSA_PIXMAP_IDENTITY,
+    "",
+    GNOME_STOCK_PIXMAP_SPELLCHECK,
+    "",
+    BALSA_PIXMAP_PRINT,
+    "",
+    GNOME_STOCK_PIXMAP_CLOSE,
+    NULL
 };
 
 static const gchar* message_toolbar[] = {
-    BALSA_PIXMAP_NEXT_UNREAD,    "",
-    GNOME_STOCK_PIXMAP_MAIL_RPL, BALSA_PIXMAP_MAIL_RPL_ALL,
-    GNOME_STOCK_PIXMAP_MAIL_FWD, "",
-    GNOME_STOCK_PIXMAP_BACK,     GNOME_STOCK_PIXMAP_FORWARD,
-    GNOME_STOCK_PIXMAP_SAVE,     "",
-    GNOME_STOCK_PIXMAP_PRINT,    "",
-    GNOME_STOCK_PIXMAP_TRASH,    NULL
+    BALSA_PIXMAP_NEXT_UNREAD,
+    "",
+    BALSA_PIXMAP_REPLY,
+    BALSA_PIXMAP_REPLY_ALL,
+    BALSA_PIXMAP_REPLY_GROUP,
+    BALSA_PIXMAP_FORWARD,
+    "",
+    BALSA_PIXMAP_PREVIOUS,
+    BALSA_PIXMAP_NEXT,
+    BALSA_PIXMAP_SAVE,
+    "",
+    BALSA_PIXMAP_PRINT,
+    "",
+    BALSA_PIXMAP_TRASH,
+    NULL
 };
 
 static const gchar* null_toolbar[] = { NULL };
 
 static void
-populate_stock_toolbar(int bar, int id)
+populate_stock_toolbar(int index, BalsaToolbarType id)
 {
     const gchar** toolbar;
     int i;
 
-    switch(bar) {
-    case 0:  toolbar = main_toolbar;    break;
-    case 1:  toolbar = compose_toolbar; break; 
-    case 2:  toolbar = message_toolbar; break;
-    default: toolbar = null_toolbar;	break;
+    switch(id) {
+    case TOOLBAR_MAIN:     toolbar = main_toolbar;    break;
+    case TOOLBAR_COMPOSE:  toolbar = compose_toolbar; break; 
+    case TOOLBAR_MESSAGE:  toolbar = message_toolbar; break;
+    default:               toolbar = null_toolbar;    break;
     }
     for(i=0; toolbar[i]; i++)
-	balsa_app.toolbars[bar][i] = g_strdup(toolbar[i]);
-    balsa_app.toolbars[bar][i]= NULL;
+	balsa_app.toolbars[index][i] = g_strdup(toolbar[i]);
+    balsa_app.toolbars[index][i]= NULL;
 }
 
 /* get_toolbar_index:
@@ -316,7 +408,7 @@ populate_stock_toolbar(int bar, int id)
    	Toolbar index in config data, or -1 if error / not found
 */
 int
-get_toolbar_index(int id)
+get_toolbar_index(BalsaToolbarType id)
 {
     int i;
     
@@ -340,7 +432,7 @@ get_toolbar_index(int id)
 	0 if OK, -1 if there are too many toolbars.
 */
 int
-create_stock_toolbar(int id)
+create_stock_toolbar(BalsaToolbarType id)
 {
     int newbar;
     
@@ -356,7 +448,6 @@ create_stock_toolbar(int id)
     balsa_app.toolbars[newbar]=
 	(char **)g_malloc(sizeof(char *)*MAXTOOLBARITEMS);
     
-    balsa_app.toolbars[newbar][0]=NULL;
     populate_stock_toolbar(newbar, id);
     balsa_app.toolbar_ids[newbar]=id;
     
@@ -449,8 +540,11 @@ get_toolbar(GtkWidget *window, BalsaToolbarType toolbar)
     for(j=0; balsa_app.toolbars[index][j]; j++) {
 	button=get_toolbar_button_index(balsa_app.toolbars[index][j]);
 	
-	if(button == -1)
+	if(button == -1) {
+	    g_warning("button '%s' not found. ABORT!\n",
+		      balsa_app.toolbars[index][j]);
 	    continue;
+	}
 
 	if(!*(balsa_app.toolbars[index][j])) {
 	    gtk_toolbar_append_space(bar);
@@ -506,7 +600,7 @@ get_toolbar(GtkWidget *window, BalsaToolbarType toolbar)
 }
 
 static int
-get_toolbar_button_slot(BalsaToolbarType toolbar, char *id)
+get_toolbar_button_slot(BalsaToolbarType toolbar, const char *id)
 {
     int i;
     
@@ -544,7 +638,7 @@ get_toolbar_button_slot(BalsaToolbarType toolbar, char *id)
 
 */
 void
-set_toolbar_button_callback(BalsaToolbarType toolbar, char *id, 
+set_toolbar_button_callback(BalsaToolbarType toolbar, const char *id, 
 			    void (*callback)(GtkWidget *, gpointer), 
 			    gpointer data)
 {
