@@ -64,55 +64,12 @@
 #define MAX(a, b) (a > b ? a : b)
 #endif
 
+#define OLD_BALSA_COMPATIBILITY_TRANSLATION
+
 static int customize_open=0;
 GtkWidget *customize_widget;
 static int word_wrap;
 
-button_data toolbar_buttons[]={
-    {"", N_("Separator"), "", 0},
-    {GNOME_STOCK_PIXMAP_MAIL_RCV, N_("Check"),
-     N_("Check for new email"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_MAIL_NEW, N_("Compose"), 
-     N_("Compose message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_MAIL, N_("Continue"), 
-     N_("Continue message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_MAIL_RPL, N_("Reply"), 
-     N_("Reply to the current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_MAIL_RPL_ALL, N_("Reply\nto all"),
-     N_("Reply to all recipients"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_MAIL_FWD, N_("Forward"), 
-     N_("Forward the current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_BACK, N_("Previous"), 
-     N_("Open previous"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_FORWARD, N_("Next"), 
-     N_("Open next"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_NEXT_UNREAD, N_("Next\nunread"), 
-     N_("Open next unread message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_TRASH, N_("Trash /\nDelete"), 
-     N_("Move the current message to trash"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_PRINT, N_("Print"), 
-     N_("Print current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_MAIL_SND, N_("Send"), 
-     N_("Send this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_ATTACH, N_("Attach"), 
-     N_("Add attachments to this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_SAVE, N_("Save"), 
-     N_("Save the current item"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_IDENTITY, N_("Select Identity"),
-     N_("Set identity to use for this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_SPELLCHECK, N_("Spelling"),
-     N_("Run a spell check"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {GNOME_STOCK_PIXMAP_CLOSE, N_("Cancel"), 
-     N_("Cancel this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_FLAG_UNREAD, N_("Toggle\nnew"), 
-     N_("Toggle new message flag"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_MARK_ALL_MSGS, N_("Mark all"),
-     N_("Mark all messages in current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_SHOW_ALL_HEADERS, N_("All\nheaders"),
-     N_("Show all headers"), TOOLBAR_BUTTON_TYPE_TOGGLE}
-};
-
-int toolbar_button_count=sizeof(toolbar_buttons)/sizeof(button_data);
 
 struct toolbar_item {
     GtkWidget *widget;
@@ -131,25 +88,28 @@ struct toolbar_page {
     int selected_destination;
 };
 
-struct toolbar_page toolbar_pages[MAXTOOLBARS];
-struct toolbar_item *toolbar_items[MAXTOOLBARS];
-int toolbar_item_count[MAXTOOLBARS];
+static struct toolbar_page toolbar_pages[STOCK_TOOLBAR_COUNT];
+static struct toolbar_item *toolbar_items[STOCK_TOOLBAR_COUNT];
+int toolbar_item_count[STOCK_TOOLBAR_COUNT];
 
-static GtkWidget *create_toolbar_page(gpointer, int);
+static GtkWidget *create_toolbar_page(BalsaToolbarType);
 static void page_destroy_cb(GtkWidget *, gpointer);
 static void populate_list(GtkWidget *, int);
 static void add_button_cb(GtkWidget *, gpointer);
 static void remove_button_cb(GtkWidget *, gpointer);
 static void back_button_cb(GtkWidget *, gpointer);
 static void forward_button_cb(GtkWidget *, gpointer);
-static void list_selected_cb(GtkCList *clist, gint row, gint column,
-			     GdkEventButton *event, gpointer user_data);
-static void destination_selected_cb(GtkCList *clist, gint row, gint column,
-				    GdkEventButton *event, gpointer user_data);
-static void recreate_preview(int toolbar, int preview_only);
-static void set_button_states(int toolbar);
-static void remove_toolbar_item(int toolbar, int item);
-static void get_toolbar_data(int toolbar);
+static void source_selected_cb(GtkCList *clist, gint row, gint column,
+                               GdkEventButton *event, gpointer user_data);
+static void source_unselected_cb(GtkCList *clist, gint row, gint column,
+                                 GdkEventButton *event, gpointer user_data);
+static void dest_selecteded_cb(GtkCList *clist, gint row, gint column,
+                               GdkEventButton *event, gpointer user_data);
+static void dest_unselected_cb(GtkCList *clist, gint row, gint column,
+                               GdkEventButton *event, gpointer user_data);
+static void recreate_preview(BalsaToolbarType toolbar, gboolean preview_only);
+static void remove_toolbar_item(BalsaToolbarType toolbar, int item);
+static void get_toolbar_data(BalsaToolbarType toolbar);
 static void apply_toolbar_prefs(GtkWidget *widget, gpointer data);
 static void wrap_toggled_cb(GtkWidget *widget, gpointer data);
 static void page_active_cb(GtkWidget *widget, GdkEvent *event, gpointer data);
@@ -167,14 +127,12 @@ page_active_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
 static void
 wrap_toggled_cb(GtkWidget *widget, gpointer data)
 {
-    int toolbar;
-    
-    toolbar=(int)data;
+    BalsaToolbarType toolbar = GPOINTER_TO_INT(data);
     
     word_wrap=GTK_TOGGLE_BUTTON(widget)->active;
     
     gnome_property_box_changed(GNOME_PROPERTY_BOX(customize_widget));
-    recreate_preview(toolbar, 0);
+    recreate_preview(toolbar, FALSE);
 }
 
 static void
@@ -182,57 +140,106 @@ apply_toolbar_prefs(GtkWidget *widget, gpointer data)
 {
     int i, j;
     int index;
-    int pos;
     
     balsa_app.toolbar_wrap_button_text=word_wrap;
     
     for(i=0; i<STOCK_TOOLBAR_COUNT; i++) {
-	pos=0;
 	index=get_toolbar_index(i);
 	if(index == -1)
 	    continue;
 	
-		for(j=0; balsa_app.toolbars[index][j]; j++) {
-		    g_free(balsa_app.toolbars[index][j]);
-		    balsa_app.toolbars[index][j]=NULL;
-		}
-		
-		for(j=0; j<toolbar_item_count[i]; j++) {
-		    balsa_app.toolbars[index][pos++]=
-			g_strdup(toolbar_buttons[toolbar_items[i][j].id].pixmap_id);
-		}
-		balsa_app.toolbars[index][pos]=NULL;
+        for(j=0; balsa_app.toolbars[index][j]; j++) {
+            g_free(balsa_app.toolbars[index][j]);
+            balsa_app.toolbars[index][j]=NULL;
+        }
+        
+        for(j=0; j<toolbar_item_count[i]; j++) {
+            balsa_app.toolbars[index][j]=
+                g_strdup(toolbar_buttons[toolbar_items[i][j].id].pixmap_id);
+        }
+        balsa_app.toolbars[index][j]=NULL;
     }
     
     update_all_toolbars();
 }
 
 /* get_toolbar_button_index:
-   FIXME: comment needed.
+   id - button id
+   returns -1 on failure.
 */
 int
-get_toolbar_button_index(char *id)
+get_toolbar_button_index(const char *id)
 {
+#ifdef OLD_BALSA_COMPATIBILITY_TRANSLATION
+    static const struct {
+        gchar *new;
+        gchar *old;
+    } button_converter[] = {
+        { BALSA_PIXMAP_ATTACHMENT,   GNOME_STOCK_PIXMAP_ATTACH },
+        { BALSA_PIXMAP_NEW,          GNOME_STOCK_PIXMAP_MAIL_NEW },
+        { BALSA_PIXMAP_CONTINUE,     GNOME_STOCK_PIXMAP_MAIL },
+        { BALSA_PIXMAP_RECEIVE,      GNOME_STOCK_PIXMAP_MAIL_RCV },
+        { BALSA_PIXMAP_REPLY,        GNOME_STOCK_PIXMAP_MAIL_RPL },
+        { BALSA_PIXMAP_REPLY_ALL,    "reply_to_all" },
+        { BALSA_PIXMAP_REPLY_GROUP,  "reply_to_group" },
+        { BALSA_PIXMAP_FORWARD,      GNOME_STOCK_PIXMAP_MAIL_FWD },
+        { BALSA_PIXMAP_NEXT,         GNOME_STOCK_PIXMAP_FORWARD },
+        { BALSA_PIXMAP_PREVIOUS,     GNOME_STOCK_PIXMAP_BACK },
+        { BALSA_PIXMAP_PRINT,        GNOME_STOCK_PIXMAP_PRINT },
+        { BALSA_PIXMAP_SAVE,         GNOME_STOCK_PIXMAP_SAVE },
+        { BALSA_PIXMAP_SEND,         GNOME_STOCK_PIXMAP_MAIL_SND },
+        { BALSA_PIXMAP_TRASH,        GNOME_STOCK_PIXMAP_TRASH },
+        { BALSA_PIXMAP_TRASH_EMPTY,  "empty_trash" },
+        { BALSA_PIXMAP_NEXT_UNREAD,  "next_unread" },
+        { BALSA_PIXMAP_NEXT_FLAGGED, "next_flagged" },
+        { BALSA_PIXMAP_SHOW_HEADERS, "show_all_headers" },
+        { BALSA_PIXMAP_SHOW_PREVIEW, "show_preview" },
+        { BALSA_PIXMAP_MARKED_NEW,   "flag_unread" },
+        { BALSA_PIXMAP_MARKED_ALL,   "mark_all" },
+        { BALSA_PIXMAP_IDENTITY,     "identity" },
+        { BALSA_PIXMAP_CLOSE_MBOX,   GNOME_STOCK_PIXMAP_CLOSE },
+        { BALSA_PIXMAP_OTHER_CLOSE,  "close_mbox" },
+        { NULL, NULL }
+    };
+#endif
     int i;
-    
+
+    g_return_val_if_fail(id, -1);
+
     for(i=0; i<toolbar_button_count; i++) {
 	if(!strcmp(id, toolbar_buttons[i].pixmap_id))
 	    return i;
     }
+#ifdef OLD_BALSA_COMPATIBILITY_TRANSLATION
+    /* you have got a second chance.... */
+    
+    for(i=0; button_converter[i].new; i++) {
+        if(!strcmp(id, button_converter[i].old)) {
+            int j;
+            for(j=0; j<toolbar_button_count; j++) {
+                if(!strcmp(button_converter[i].new,
+                           toolbar_buttons[j].pixmap_id))
+                    return j;
+            }
+            return -1;
+        }
+    }
+#endif
     return -1;
 }
 
 static void
-get_toolbar_data(int toolbar)
+get_toolbar_data(BalsaToolbarType toolbar)
 {
     int index;
     int button;
     int i;
     
     index=get_toolbar_index(toolbar);
+    g_return_if_fail(index>=0);
     toolbar_item_count[toolbar]=0;
     
-    for(i=0; balsa_app.toolbars[toolbar][i]; i++) {
+    for(i=0; balsa_app.toolbars[index][i]; i++) {
 	button=get_toolbar_button_index(balsa_app.toolbars[index][i]);
 	if(button == -1) {
 	    printf("Warning: unknown button %s\n",
@@ -244,31 +251,6 @@ get_toolbar_data(int toolbar)
     }
 }
 
-static void
-set_button_states(int toolbar)
-{
-    if(toolbar_pages[toolbar].selected_destination == -1) {
-	gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, FALSE);
-	gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, FALSE);
-	gtk_widget_set_sensitive(toolbar_pages[toolbar].remove_button, FALSE);
-	return;
-    }
-    gtk_widget_set_sensitive(toolbar_pages[toolbar].remove_button, TRUE);
-    
-    if(toolbar_item_count[toolbar] <= 1) {
-	gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, FALSE);
-	gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, FALSE);
-	return;
-    }
-    
-    gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, 
-			     toolbar_pages[toolbar].selected_destination > 0);
-    
-    gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, 
-			     toolbar_pages[toolbar].selected_destination <
-			     toolbar_item_count[toolbar]-1);
-}
-
 static void
 remove_all_buttons(GtkToolbar *bar)
 {
@@ -300,7 +282,7 @@ replace_nl_with_space(char* str)
     }
 }
 static void
-recreate_preview(int toolbar, int preview_only)
+recreate_preview(BalsaToolbarType toolbar, gboolean preview_only)
 {
     GtkToolbar *bar;
     GtkWidget *btn;
@@ -378,7 +360,6 @@ recreate_preview(int toolbar, int preview_only)
 			       gtk_clist_optimal_column_width(
 				   GTK_CLIST(toolbar_pages[toolbar].destination), 1));
     
-    set_button_states(toolbar);
     gtk_widget_show_all(toolbar_pages[toolbar].preview);
     
 }
@@ -406,7 +387,7 @@ back_button_cb(GtkWidget *widget, gpointer data)
     toolbar_items[toolbar][toolbar_pages[toolbar].selected_destination-1]=tmp;
     --toolbar_pages[toolbar].selected_destination;
     
-    recreate_preview(toolbar, 1);
+    recreate_preview(toolbar, TRUE);
     gtk_clist_swap_rows(GTK_CLIST(toolbar_pages[toolbar].destination),
 			row, row-1);
     gtk_clist_moveto(GTK_CLIST(toolbar_pages[toolbar].destination),
@@ -437,7 +418,7 @@ forward_button_cb(GtkWidget *widget, gpointer data)
     toolbar_items[toolbar][toolbar_pages[toolbar].selected_destination+1]=tmp;
     ++toolbar_pages[toolbar].selected_destination;
     
-    recreate_preview(toolbar, 1);
+    recreate_preview(toolbar, TRUE);
     gtk_clist_swap_rows(GTK_CLIST(toolbar_pages[toolbar].destination),
 			row, row+1);
     gtk_clist_moveto(GTK_CLIST(toolbar_pages[toolbar].destination),
@@ -446,7 +427,7 @@ forward_button_cb(GtkWidget *widget, gpointer data)
 }
 
 static void
-remove_toolbar_item(int toolbar, int item)
+remove_toolbar_item(BalsaToolbarType toolbar, int item)
 {
     int i;
     
@@ -469,40 +450,59 @@ remove_button_cb(GtkWidget *widget, gpointer data)
     pos=toolbar_pages[toolbar].selected_destination;
     remove_toolbar_item(toolbar, pos);
     populate_list(toolbar_pages[toolbar].list, toolbar);
-    recreate_preview(toolbar, 1);
     gtk_clist_remove(list, toolbar_pages[toolbar].selected_destination);
     gnome_property_box_changed(GNOME_PROPERTY_BOX(customize_widget));
+    recreate_preview(toolbar, TRUE);
 }
 
 static void
-list_selected_cb(GtkCList *clist, gint row, gint column,
-		 GdkEventButton *event, gpointer user_data)
+source_selected_cb(GtkCList *clist, gint row, gint column,
+                   GdkEventButton *event, gpointer user_data)
 {
-    int toolbar;
-
-    toolbar=(int)user_data;
+    int toolbar = GPOINTER_TO_INT(user_data);
     toolbar_pages[toolbar].selected_source=row;
     gtk_widget_set_sensitive(toolbar_pages[toolbar].add_button, TRUE);
 }
+static void
+source_unselected_cb(GtkCList *clist, gint row, gint column,
+                     GdkEventButton *event, gpointer user_data)
+{
+    int toolbar = GPOINTER_TO_INT(user_data);
+    toolbar_pages[toolbar].selected_source=-1;
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].add_button, FALSE);
+}
 
 static void
-destination_selected_cb(GtkCList *clist, gint row, gint column,
-			GdkEventButton *event, gpointer user_data)
+dest_selected_cb(GtkCList *clist, gint row, gint column,
+                 GdkEventButton *event, gpointer user_data)
 {
-    int toolbar;
-    
-    toolbar=(int)user_data;
+    int toolbar = GPOINTER_TO_INT(user_data);
     
     toolbar_pages[toolbar].selected_destination=row;
-    gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, TRUE);
-    gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, TRUE);
-    gtk_widget_set_sensitive(toolbar_pages[toolbar].remove_button, TRUE);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].remove_button,  TRUE);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, 
+			     toolbar_pages[toolbar].selected_destination > 0);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, 
+			     toolbar_pages[toolbar].selected_destination <
+			     toolbar_item_count[toolbar]-1);
+}
+
+static void
+dest_unselected_cb(GtkCList *clist, gint row, gint column,
+                   GdkEventButton *event, gpointer user_data)
+{
+    int toolbar = GPOINTER_TO_INT(user_data);
+    
+    toolbar_pages[toolbar].selected_destination=-1;
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].remove_button,  FALSE);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button,    FALSE);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, FALSE);
 }
 
 static void
 add_button_cb(GtkWidget *widget, gpointer data)
 {
-    int toolbar;
+    BalsaToolbarType toolbar;
     int add_item;
     GtkCList *list;
     int row;
@@ -511,7 +511,7 @@ add_button_cb(GtkWidget *widget, gpointer data)
     char *text, *wrap;
     int i;
 
-    toolbar=(int)data;
+    toolbar=GPOINTER_TO_INT(data);
     list_data[0]=list_data[1]=NULL;
 
     list=GTK_CLIST(toolbar_pages[toolbar].destination);
@@ -567,11 +567,9 @@ add_button_cb(GtkWidget *widget, gpointer data)
     gtk_clist_select_row(list, row, 0);
     gtk_clist_moveto(list, row, 0, 0, 0);
 
-    recreate_preview(toolbar, 1);
-    set_button_states(toolbar);
     gtk_widget_show_all(toolbar_pages[toolbar].preview);
-    set_button_states(toolbar);
     gnome_property_box_changed(GNOME_PROPERTY_BOX(customize_widget));
+    recreate_preview(toolbar, TRUE);
 }
 
 static void
@@ -586,6 +584,8 @@ populate_list(GtkWidget *list, int toolbar)
 
     legal=get_legal_toolbar_buttons(toolbar);
 
+    /* FIXME: clear does not emit unselect-row signal for selected rows? */
+    gtk_clist_unselect_all(GTK_CLIST(list)); 
     gtk_clist_clear(GTK_CLIST(list));
 
     gtk_clist_set_row_height(GTK_CLIST(list), 24);
@@ -635,7 +635,7 @@ page_destroy_cb(GtkWidget *widget, gpointer data)
 {
     int toolbar;
 
-    toolbar=(int)data;
+    toolbar=GPOINTER_TO_INT(data);
     g_free(toolbar_items[toolbar]);
     toolbar_item_count[toolbar]=0;
     customize_open=0;
@@ -672,22 +672,22 @@ customize_dialog_cb(GtkWidget *widget, gpointer data)
     word_wrap=balsa_app.toolbar_wrap_button_text;
     
     gnome_property_box_append_page(GNOME_PROPERTY_BOX(customize_widget),
-				   create_toolbar_page(customize_widget, 0),
+                                   create_toolbar_page(TOOLBAR_MAIN),
 				   gtk_label_new(_("Main window")));
     
     gnome_property_box_append_page(GNOME_PROPERTY_BOX(customize_widget),
-				   create_toolbar_page(customize_widget, 1),
+                                   create_toolbar_page(TOOLBAR_COMPOSE),
 				   gtk_label_new(_("Compose window")));
     
     gnome_property_box_append_page(GNOME_PROPERTY_BOX(customize_widget),
-				   create_toolbar_page(customize_widget, 2),
+                                   create_toolbar_page(TOOLBAR_MESSAGE),
 				   gtk_label_new(_("Message window")));
     
     gtk_widget_show_all(customize_widget);
 }
 
 static GtkWidget*
-create_toolbar_page(gpointer data, int toolbar)
+create_toolbar_page(BalsaToolbarType toolbar)
 {
     GtkWidget *outer_box;
     GtkWidget *preview_frame, *preview_scroll, *preview_box;
@@ -764,7 +764,6 @@ create_toolbar_page(gpointer data, int toolbar)
 
     /* A CList to show the available items */
     list_scroll=gtk_scrolled_window_new(NULL, NULL);
-    /* gtk_widget_set_usize(list_scroll, 200, 200); */
 
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(list_scroll),
 				   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
@@ -854,10 +853,14 @@ create_toolbar_page(gpointer data, int toolbar)
 
     /* UI signals */
     gtk_signal_connect(GTK_OBJECT(list), "select-row",
-		       list_selected_cb, (gpointer)toolbar);
+		       source_selected_cb, (gpointer)toolbar);
+    gtk_signal_connect(GTK_OBJECT(list), "unselect-row",
+		       source_unselected_cb, (gpointer)toolbar);
 
     gtk_signal_connect(GTK_OBJECT(destination), "select-row",
-		       destination_selected_cb, (gpointer)toolbar);
+		       dest_selected_cb, (gpointer)toolbar);
+    gtk_signal_connect(GTK_OBJECT(destination), "unselect-row",
+		       dest_unselected_cb, (gpointer)toolbar);
 
     gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
 		       add_button_cb, (gpointer)toolbar);
@@ -884,7 +887,7 @@ create_toolbar_page(gpointer data, int toolbar)
 
     populate_list(list, toolbar);
 
-    recreate_preview(toolbar, 0);
+    recreate_preview(toolbar, FALSE);
 
     return outer_box;
 }
